@@ -107,7 +107,7 @@
 
 (defconstant digit-size sb-vm:n-word-bits)
 
-(defconstant all-ones-digit (1- (ash 1 sb-vm:n-word-bits)))
+(defconstant all-ones-digit most-positive-word)
 
 ;;;; internal inline routines
 
@@ -138,7 +138,7 @@
   (declare (type bignum-element-type digit))
   (not (logbitp (1- digit-size) digit)))
 
-#!-sb-fluid (declaim (inline %bignum-0-or-plusp))
+#-sb-fluid (declaim (inline %bignum-0-or-plusp))
 (defun %bignum-0-or-plusp (bignum len)
   (declare (type bignum bignum)
            (type bignum-length len))
@@ -207,15 +207,8 @@
 
 ;;; This takes three digits and returns the FLOOR'ed result of
 ;;; dividing the first two as a 2*digit-size integer by the third.
-;;;
-;;; Do weird LET and SETQ stuff to bamboozle the compiler into allowing
-;;; the %BIGFLOOR transform to expand into pseudo-assembler for which the
-;;; compiler can later correctly allocate registers.
 (defun %bigfloor (a b c)
-  (let ((a a) (b b) (c c))
-    (declare (type bignum-element-type a b c))
-    (setq a a b b c c)
-    (%bigfloor a b c)))
+  (%bigfloor a b c))
 
 ;;; Convert the digit to a regular integer assuming that the digit is signed.
 (defun %fixnum-digit-with-correct-sign (digit)
@@ -255,7 +248,7 @@
 ;;; is suitable for infinite sign extension to complete additions,
 ;;; subtractions, negations, etc. This cannot return a -1 represented as
 ;;; a negative fixnum since it would then have to low zeros.
-#!-sb-fluid (declaim (inline %sign-digit))
+#-sb-fluid (declaim (inline %sign-digit))
 (defun %sign-digit (bignum len)
   (declare (type bignum bignum)
            (type bignum-length len))
@@ -575,7 +568,7 @@
         (umask 0)
         (imask 1)
         (m 0))
-    (declare (type (unsigned-byte #.sb-vm:n-word-bits) ud vd umask imask m))
+    (declare (type word ud vd umask imask m))
     (dotimes (i digit-size)
       (setf umask (logior umask imask))
       (when (logtest ud umask)
@@ -605,7 +598,7 @@
                     (bignum-buffer-integer-length v v-len))))
          (n (1- (ash 1 d))))
     (declare (type (unsigned-byte #.(integer-length #.sb-vm:n-word-bits)) d)
-             (type (unsigned-byte #.sb-vm:n-word-bits) n))
+             (type word n))
     (gcd-assert (>= d 0))
     (when (logtest (%bignum-ref u 0) n)
       (let ((tmp1-len
@@ -629,7 +622,7 @@
          (d1 1)
          (n2 (modularly (1+ (modularly (lognot n1)))))
          (d2 (modularly -1)))
-    (declare (type (unsigned-byte #.sb-vm:n-word-bits) n1 d1 n2 d2))
+    (declare (type word n1 d1 n2 d2))
     (loop while (> n2 (expt 2 (truncate digit-size 2))) do
           (loop for i of-type (mod #.sb-vm:n-word-bits)
                 downfrom (- (integer-length n1) (integer-length n2))
@@ -642,19 +635,18 @@
                  n2 n1
                  d2 d1))
     (values n2 (if (>= d2 (expt 2 (1- digit-size)))
-                   (lognot (logand most-positive-fixnum (lognot d2)))
+                   (lognot (logand sb-xc:most-positive-fixnum (lognot d2)))
                    (logand lower-ones-digit d2)))))
 
 
 (defun copy-bignum (a &optional (len (%bignum-length a)))
   (let ((b (%allocate-bignum len)))
     (bignum-replace b a)
-    (%bignum-set-length b len)
     b))
 
 ;;; Allocate a single word bignum that holds fixnum. This is useful when
 ;;; we are trying to mix fixnum and bignum operands.
-#!-sb-fluid (declaim (inline make-small-bignum))
+#-sb-fluid (declaim (inline make-small-bignum))
 (defun make-small-bignum (fixnum)
   (let ((res (%allocate-bignum 1)))
     (setf (%bignum-ref res 0) (%fixnum-to-digit fixnum))
@@ -1160,6 +1152,7 @@
 
 ;;; Make a single or double float with the specified significand,
 ;;; exponent and sign.
+;;; FIXME: how are these not the same as {SINGLE,DOUBLE}-FROM-BITS ???
 (defun single-float-from-bits (bits exp plusp)
   (declare (fixnum exp))
   ;; "float to pointer coercion -> return value"
@@ -1188,7 +1181,7 @@
                            hi
                            (logior hi (ash -1 sb-vm:float-sign-shift)))
                        lo)))
-#!+(and long-float x86)
+#+(and long-float x86)
 (defun long-float-from-bits (bits exp plusp)
   (declare (fixnum exp))
   (make-long-float
@@ -1234,7 +1227,7 @@
                    (check-exponent len sb-vm:double-float-bias
                                    sb-vm:double-float-normal-exponent-max)
                    plusp))
-                 #!+long-float
+                 #+long-float
                  (long-float
                   (long-float-from-bits
                    bits
@@ -1888,7 +1881,7 @@
 ;;; incoming data, such as in-place shifting. This is basically the same as
 ;;; the first form in %NORMALIZE-BIGNUM, but we return the length of the buffer
 ;;; instead of shrinking the bignum.
-#!-sb-fluid (declaim (maybe-inline %normalize-bignum-buffer))
+#-sb-fluid (declaim (maybe-inline %normalize-bignum-buffer))
 (defun %normalize-bignum-buffer (result len)
   (declare (type bignum result)
            (type bignum-length len))
@@ -1914,7 +1907,7 @@
   (declare (type bignum result)
            (type bignum-length len)
            (muffle-conditions compiler-note)
-           #!-sb-fluid (inline %normalize-bignum-buffer))
+           #-sb-fluid (inline %normalize-bignum-buffer))
   (let ((newlen (%normalize-bignum-buffer result len)))
     (declare (type bignum-length newlen))
     (unless (= newlen len)
@@ -1933,7 +1926,7 @@
 (defun %mostly-normalize-bignum (result len)
   (declare (type bignum result)
            (type bignum-length len)
-           #!-sb-fluid (inline %normalize-bignum-buffer))
+           #-sb-fluid (inline %normalize-bignum-buffer))
   (let ((newlen (%normalize-bignum-buffer result len)))
     (declare (type bignum-length newlen))
     (unless (= newlen len)
@@ -1950,7 +1943,7 @@
       (declare (type index i))
       (let ((xi (%bignum-ref x i)))
         (mixf result
-              (logand most-positive-fixnum
+              (logand sb-xc:most-positive-fixnum
                       (logxor xi
                               (ash xi -7))))))
     result))
