@@ -7,11 +7,11 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!PCL")
+(in-package "SB-PCL")
 
 ;;;; Rudimentary DEFMETHOD
 
-(sb!xc:defmacro defmethod (&whole form name lambda-list &rest body
+(sb-xc:defmacro defmethod (&whole form name lambda-list &rest body
                            &aux qualifier)
   (when (member name '((setf documentation) documentation) :test 'equal)
     (return-from defmethod `(push ',form *!documentation-methods*)))
@@ -34,7 +34,9 @@
              (unspecialized-ll `(,(caar lambda-list) ,@(cdr lambda-list)))
              ((forms decls) (parse-body body nil))) ; Note: disallowing docstring
     `(!trivial-defmethod
-      ',name ',specializer ,qualifier ',unspecialized-ll
+      ;; An extra NIL in front puts the GF name is the same position it would be in
+      ;; for a normal LOAD-DEFMETHOD.
+      nil ',name ',specializer ,qualifier ',unspecialized-ll
       ;; OAOO problem: compute the same lambda name as real DEFMETHOD would
       (named-lambda (fast-method ,name
                      (,specializer ,@(if (eq name 'print-object) '(t))))
@@ -54,37 +56,63 @@
             (declare (inline (setf slot-value)))
             (block ,name ,@forms))))
       ;; Why is SOURCE-LOC needed? Lambdas should know their location.
-      (sb!c::source-location))))
+      (sb-c::source-location))))
 
 (defvar *!trivial-methods* '()) ; necessary methods for system startup
 (defvar *!documentation-methods* nil) ; saved up for after PCL bootstrap
-(defun !trivial-defmethod (name specializer qualifier lambda-list lambda source-loc)
+(defun !trivial-defmethod (dummy name specializer qualifier lambda-list lambda source-loc)
+  (declare (ignore dummy)) ; this would be the method class in LOAD-DEFMETHOD
   (let ((gf (assoc name *!trivial-methods*)))
     ;; Append the method but don't bother finding a predicate for it.
     ;; Methods occurring in early warm load (notably from SB-FASTEVAL)
-    ;; wil be properly installed when 'pcl/print-object.lisp' is loaded.
+    ;; will be properly installed when 'pcl/print-object.lisp' is loaded.
     (rplacd gf (concatenate 'vector (cdr gf)
-                            (list (list nil lambda specializer qualifier
-                                        lambda-list source-loc))))))
+                            (list (vector nil ; guard
+                                          qualifier specializer lambda
+                                          lambda-list source-loc))))))
 
 ;;; Slow-but-correct logic for single-dispatch sans method combination,
 ;;; allowing exactly one primary method. Methods are sorted most-specific-first,
 ;;; so we can stop looking as soon as a match is found.
+;;; Sorting is performed during genesis.
 (defun !call-a-method (gf-name specialized-arg &rest rest)
   (let* ((methods (the simple-vector
                     (cdr (or (assoc gf-name *!trivial-methods*)
                              (error "No methods on ~S" gf-name)))))
          (applicable-method
-          (find specialized-arg methods
-                :test (lambda (arg method &aux (guard (car method)))
-                        (and (or (functionp guard) (fboundp guard))
-                             (funcall guard arg))))))
-    (unless applicable-method
-      (error "No applicable method for ~S on ~S~%" gf-name
-             (type-of specialized-arg)))
-    ;; The "method" is a list: (GUARD LAMBDA . OTHER-STUFF)
-    ;; Call using no permutation-vector / no precomputed next method.
-    (apply (cadr applicable-method) nil nil specialized-arg rest)))
+           ;; Find a method where the guard returns T. If that fails, find a method
+           ;; which exactly matches TYPE-OF the specialized-arg.
+           ;; It might be nice to rely only on the TYPE-OF test, but then we'd have to
+           ;; concern ourselves with type hierarchies.
+           ;; The "method" is a vector:
+           ;;  #(#<GUARD> QUALIFIER SPECIALIZER #<FMF> LAMBDA-LIST SOURCE-LOC)
+           (or (find-if (lambda (method &aux (guard (svref method 0)))
+                          (and (or (functionp guard) (fboundp guard))
+                               (funcall guard specialized-arg)))
+                        methods)
+               (find (type-of specialized-arg) methods
+                     :key (lambda (x) (svref x 2))))))
+
+    (if applicable-method
+        ;; Call using no permutation-vector / no precomputed next method.
+        (apply (svref applicable-method 3) nil nil specialized-arg rest)
+        (error "No applicable method for ~S on ~S~%" gf-name
+               (type-of specialized-arg)))))
+
+;;; For any "trivial" PRINT-OBJECT method lacking a predicate to determine
+;;; its applicability, try to install one now.
+;;; (Question: could not genesis find a predicate for everything?)
+(defun !fixup-print-object-method-guards ()
+  ;; LAYOUT-OF has an ordinary definition appearing later, unless a vop translates
+  ;; it, in which case it has to be inlined because the stub isn't ready for use.
+  #-(vop-translates sb-kernel:layout-of) (declare (notinline layout-of))
+  ;; Wouldn't you know, DOVECTOR is in early-extensions,
+  ;; and it would need to go in primordial-extensions to use it here.
+  (loop for method across (cdr (assoc 'print-object sb-pcl::*!trivial-methods*))
+        unless (svref method 0)
+        do (let ((classoid (find-classoid (elt method 2))))
+             (setf (svref method 0)
+                   (lambda (x) (classoid-typep (layout-of x) classoid x))))))
 
 (defun make-load-form (object &optional environment)
   (!call-a-method 'make-load-form object environment))
@@ -94,3 +122,9 @@
 ;;; FIXME: this no longer holds methods, but it seems to have an effect
 ;;; on the caching of a discriminating function for PRINT-OBJECT
 (defvar *!delayed-defmethod-args* nil)
+
+;;; This exists only to show that the cross-compiler can constant-fold
+;;; a constant index into a literal array without crashing.
+(defun !test-svref-folding ()
+  (let ((z #(42 test)))
+    (if (< (svref z 0) 100) t nil)))
