@@ -631,18 +631,31 @@ exist or if is a file or a symbolic link."
           (delete-dir physical)))))
 
 
-(sb-alien:define-alien-variable ("sbcl_home" *sbcl-home*) c-string)
+(sb-alien:define-alien-variable ("runtime_home" *runtime-home*) c-string)
 
 (defun sbcl-homedir-pathname ()
   (let ((env (posix-getenv "SBCL_HOME")))
     ;; Should we absoluteize this if it was obtained automatically?
     ;; Depends whether people are in the habit of using chdir within Lisp.
-    (parse-native-namestring (if (and env (not (string= env "")))
-                                 env
-                                 (or *sbcl-home* ""))
-                             *physical-host*
-                             *default-pathname-defaults*
-                             :as-directory t)))
+    (flet ((parse (namestring)
+             (parse-native-namestring namestring
+                                      *physical-host*
+                                      *default-pathname-defaults*
+                                      :as-directory t))
+           (probe (path)
+             (when (probe-file (merge-pathnames "contrib/"
+                                                path))
+               path)))
+      (let ((env (and env (not (string= env ""))
+                      (parse env)))
+            (parsed-runtime-home))
+        (or (and env
+                 (probe env))
+            (probe (setf parsed-runtime-home
+                         (parse (or *runtime-home* ""))))
+            (probe (merge-pathnames "../lib/sbcl/"
+                                    parsed-runtime-home))
+            (parse "."))))))
 
 (flet ((not-empty (x)
          (and (not (equal x "")) x))
@@ -990,7 +1003,7 @@ Experimental: interface subject to change."
            #'cont
            (lambda (sub)
              (when (pattern-matches this (last-directory-piece sub))
-               (funcall #'cont sub))))
+               (cont sub))))
        directory
        :files nil
        :directories t
