@@ -512,9 +512,9 @@
 ;;; putting the implementation and version in required fields in the
 ;;; fasl file header.)
 
-;; Cold-load calls COLD-LOAD-CODE instead
 (define-fop 16 :not-host (fop-load-code ((:operands header n-code-bytes n-fixups)))
-  (let* ((n-boxed-words (ash header -1))
+  (let* ((n-named-calls (read-unsigned-byte-32-arg (fasl-input-stream)))
+         (n-boxed-words (ash header -1))
          (n-constants (- n-boxed-words sb-vm:code-constants-offset)))
     ;; stack has (at least) N-CONSTANTS words plus debug-info
     (with-fop-stack ((stack (operand-stack)) ptr (1+ n-constants))
@@ -522,17 +522,21 @@
              (n-boxed-words (+ sb-vm:code-constants-offset n-constants))
              (code (sb-c:allocate-code-object
                     (if (oddp header) :immobile :dynamic)
+                    n-named-calls
                     (align-up n-boxed-words sb-c::code-boxed-words-align)
                     n-code-bytes)))
-        (setf (%code-debug-info code) (svref stack debug-info-index))
         (loop for i of-type index from sb-vm:code-constants-offset
               for j of-type index from ptr below debug-info-index
               do (setf (code-header-ref code i) (svref stack j)))
         (with-pinned-objects (code)
-          ;; * DO * NOT * SEPARATE * THESE * TWO * STEPS *
+          ;; * DO * NOT * SEPARATE * THESE * STEPS *
           ;; For a full explanation, refer to the comment above MAKE-CORE-COMPONENT
-          ;; concerning the corresponding use therein of WITH-PINNED-OBJECTS.
+          ;; concerning the corresponding use therein of WITH-PINNED-OBJECTS etc.
           (read-n-bytes (fasl-input-stream) (code-instructions code) 0 n-code-bytes)
+          (sb-thread:barrier (:write))
+          ;; Assign debug-info last. A code object that has no debug-info will never
+          ;; have its fun table accessed in conservative_root_p() or pin_object().
+          (setf (%code-debug-info code) (svref stack debug-info-index))
           (sb-c::apply-fasl-fixups stack code n-fixups))
         #-sb-xc-host
         (when (typep (code-header-ref code (1- n-boxed-words))
