@@ -463,6 +463,7 @@ HOLDING-MUTEX-P."
   %owner
   %head
   %tail)
+(declaim (sb-ext:freeze-type waitqueue))
 
 ;;; Signals an error if owner of LOCK is waiting on a lock whose release
 ;;; depends on the current thread. Does not detect deadlocks from sempahores.
@@ -750,6 +751,7 @@ returns NIL each time."
           (multiple-value-call #'%wait-for-mutex
             mutex new-owner timeout (decode-timeout timeout))))))
 
+(declaim (ftype (sfunction (mutex &key (:waitp t) (:timeout (or null (real 0)))) boolean) grab-mutex))
 (defun grab-mutex (mutex &key (waitp t) (timeout nil))
   "Acquire MUTEX for the current thread. If WAITP is true (the default) and
 the mutex is not immediately available, sleep until it is available.
@@ -793,6 +795,7 @@ Notes:
           (multiple-value-call #'%wait-for-mutex
             mutex self timeout (decode-timeout timeout))))))
 
+(declaim (ftype (sfunction (mutex &key (:if-not-owner (member :punt :warn :error :force))) null) release-mutex))
 (defun release-mutex (mutex &key (if-not-owner :punt))
   "Release MUTEX by setting it to NIL. Wake up threads waiting for
 this mutex.
@@ -813,6 +816,8 @@ IF-NOT-OWNER is :FORCE)."
         ((:punt) (return-from release-mutex nil))
         ((:warn)
          (warn "Releasing ~S, owned by another thread: ~S" mutex old-owner))
+        ((:error)
+         (error "Releasing ~S, owned by another thread: ~S" mutex old-owner))
         ((:force)))
       (setf (mutex-%owner mutex) nil)
       ;; FIXME: Is a :memory barrier too strong here?  Can we use a :write
@@ -1011,6 +1016,7 @@ IF-NOT-OWNER is :FORCE)."
          (bug "%CONDITION-WAIT: invalid status on normal return: ~S" status))))))
 (declaim (notinline %condition-wait))
 
+(declaim (ftype (sfunction (waitqueue mutex &key (:timeout (or null (real 0)))) boolean) condition-wait))
 (defun condition-wait (queue mutex &key timeout)
   "Atomically release MUTEX and start waiting on QUEUE until another thread
 wakes us up using either CONDITION-NOTIFY or CONDITION-BROADCAST on
@@ -1050,7 +1056,6 @@ associated data:
       (push data *data*)
       (condition-notify *queue*)))
 "
-  (assert mutex)
   (locally (declare (inline %condition-wait))
     (multiple-value-bind (to-sec to-usec stop-sec stop-usec deadlinep)
         (decode-timeout timeout)
@@ -1058,6 +1063,8 @@ associated data:
        (%condition-wait queue mutex timeout
                         to-sec to-usec stop-sec stop-usec deadlinep)))))
 
+(declaim (ftype (sfunction (waitqueue &optional (and fixnum (integer 1))) null)
+                condition-notify))
 (defun condition-notify (queue &optional (n 1))
   "Notify N threads waiting on QUEUE.
 
@@ -1068,26 +1075,27 @@ must be held by this thread during this call."
   #-sb-thread
   (error "Not supported in unithread builds.")
   #+sb-thread
-  (declare (type (and fixnum (integer 1)) n))
-  #+sb-thread
   (progn
     #-sb-futex
     (with-cas-lock ((waitqueue-%owner queue))
       (%waitqueue-wakeup queue n))
     #+sb-futex
     (progn
-    ;; No problem if >1 thread notifies during the comment in condition-wait:
-    ;; as long as the value in queue-data isn't the waiting thread's id, it
-    ;; matters not what it is -- using the queue object itself is handy.
-    ;;
-    ;; XXX we should do something to ensure that the result of this setf
-    ;; is visible to all CPUs.
-    ;;
-    ;; ^-- surely futex_wake() involves a memory barrier?
+      ;; No problem if >1 thread notifies during the comment in condition-wait:
+      ;; as long as the value in queue-data isn't the waiting thread's id, it
+      ;; matters not what it is -- using the queue object itself is handy.
+      ;;
+      ;; XXX we should do something to ensure that the result of this setf
+      ;; is visible to all CPUs.
+      ;;
+      ;; ^-- surely futex_wake() involves a memory barrier?
       (setf (waitqueue-token queue) queue)
       (with-pinned-objects (queue)
-        (futex-wake (waitqueue-token-address queue) n)))))
+        (futex-wake (waitqueue-token-address queue) n))
+      nil)))
 
+
+(declaim (ftype (sfunction (waitqueue) null) condition-broadcast))
 (defun condition-broadcast (queue)
   "Notify all threads waiting on QUEUE.
 
@@ -1111,8 +1119,11 @@ future."
   (name    nil :type (or null string) :read-only t)
   (%count    0 :type (integer 0))
   (waitcount 0 :type sb-vm:word)
-  (mutex (make-mutex :name "semaphore lock") :read-only t)
-  (queue (make-waitqueue) :read-only t))
+  (mutex (make-mutex :name "semaphore lock") :read-only t
+                                             :type mutex)
+  (queue (make-waitqueue) :read-only t
+                          :type waitqueue))
+(declaim (sb-ext:freeze-type semaphore))
 
 (setf (documentation 'semaphore-name 'function)
       "The name of the semaphore INSTANCE. Setfable."
@@ -1125,6 +1136,7 @@ future."
 TRY-SEMAPHORE as the :NOTIFICATION argument. Consequences are undefined if
 multiple threads are using the same notification object in parallel."
   (%status nil :type boolean))
+(declaim (sb-ext:freeze-type semaphore-notification))
 
 (setf (documentation 'make-semaphore-notification 'function)
       "Constructor for SEMAPHORE-NOTIFICATION objects. SEMAPHORE-NOTIFICATION-STATUS
@@ -1253,6 +1265,7 @@ with SEMAPHORE-NOTIFICATION-STATUS of NIL. If the count is decremented,
 the status is set to T."
   (%decrement-semaphore semaphore n nil notification 'try-semaphore))
 
+(declaim (ftype (sfunction (semaphore &optional (integer 1)) null) signal-semaphore))
 (defun signal-semaphore (semaphore &optional (n 1))
   "Increment the count of SEMAPHORE by N. If there are threads waiting
 on this semaphore, then N of them is woken up."
@@ -1273,6 +1286,7 @@ on this semaphore, then N of them is woken up."
   (threads nil)
   (interactive-threads nil)
   (interactive-threads-queue (make-waitqueue)))
+(declaim (sb-ext:freeze-type session))
 
 (defvar *session* nil)
 
@@ -1597,7 +1611,7 @@ session."
                                      sb-vm:*control-stack-start*)))))))))
   (values))
 
-(defun make-thread (function &key name arguments ephemeral)
+(defun make-thread (function &key name arguments)
   "Create a new thread of NAME that runs FUNCTION with the argument
 list designator provided (defaults to no argument). Thread exits when
 the function returns. The return values of FUNCTION are kept around
@@ -1607,7 +1621,7 @@ Invoking the initial ABORT restart established by MAKE-THREAD
 terminates the thread.
 
 See also: RETURN-FROM-THREAD, ABORT-THREAD."
-  #-sb-thread (declare (ignore function name arguments ephemeral))
+  #-sb-thread (declare (ignore function name arguments))
   #-sb-thread (error "Not supported in unithread builds.")
   #+sb-thread
   (progn (assert (or (atom arguments)
@@ -1615,8 +1629,12 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
                  (arguments)
                  "Argument passed to ~S, ~S, is an improper list."
                  'make-thread arguments)
-         (run-thread (%make-thread :name name :%ephemeral-p ephemeral)
-                     function arguments)))
+         (run-thread (%make-thread :name name) function arguments)))
+
+;;; System-internal use only
+#+sb-thread
+(defun make-ephemeral-thread (name function arguments)
+  (run-thread (%make-thread :name name :%ephemeral-p t) function arguments))
 
 ;;; The purpose of splitting out RUN-THREAD from MAKE-THREAD is that when
 ;;; starting the finalizer thread, we might be able to do:
@@ -2073,8 +2091,8 @@ mechanism for inter-thread communication."
 
 (eval-when (:compile-toplevel)
   ;; Inform genesis of the index <-> symbol mapping made by DEFINE-THREAD-LOCAL
-  (with-open-file (output (sb-cold::stem-object-path "tls-init.lisp-expr"
-                                                     '(:extra-artifact) :target-compile)
+  (with-open-file (output (sb-cold:stem-object-path "tls-init.lisp-expr"
+                                                    '(:extra-artifact) :target-compile)
                           :direction :output :if-exists :supersede)
     (let ((list (mapcar (lambda (x &aux (symbol (car x)))
                           (cons (info :variable :wired-tls symbol) symbol))
@@ -2124,7 +2142,9 @@ mechanism for inter-thread communication."
                        (*print-lines* 4))
                    (format t " ~3d ~30a : ~s~%"
                            (ash tlsindex (- sb-vm:word-shift))
-                           (sb-ext::find-symbol-from-tls-index tlsindex)
+                           ;; FIND-SYMBOL-FROM-TLS-INDEX uses MAP-ALLOCATED-OBJECTS
+                           ;; which is not defined during cross-compilation.
+                           (funcall 'sb-ext::find-symbol-from-tls-index tlsindex)
                            val)))))
       (format t "~&TLS: (base=~x)~%" (sap-int sap))
       (loop for tlsindex from sb-vm:n-word-bytes below
