@@ -18,7 +18,7 @@
             composite-immediate-instruction encodable-immediate
             lsl lsr asr ror cpsr @) "SB-VM")
   ;; Imports from SB-VM into this package
-  (import '(sb-vm::nil-value sb-vm::registers sb-vm::null-tn sb-vm::null-offset
+  (import '(sb-vm:nil-value sb-vm::registers sb-vm::null-tn sb-vm::null-offset
             sb-vm::pc-tn sb-vm::pc-offset sb-vm::code-offset)))
 
 
@@ -152,6 +152,33 @@
   (rn :field (byte 4 16) :type 'reg)
   (rd :field (byte 4 12) :type 'reg))
 
+;;; Not sure if we can coerce our disassembler to print anything resembling this:
+;;; <LDM|STM>{cond}<FD|ED|FA|EA|IA|IB|DA|DB> Rn{!},<Rlist>{^} where:
+;;;   {cond}  two-character condition mnemonic. See Table 4-2: Condition code
+;;;           summary on page 4-5.
+;;;   Rn      is an expression evaluating to a valid register number
+;;;   <Rlist> is a list of registers and register ranges enclosed in {} (For example,
+;;;           {R0,R2-R7,R10}).
+;;;   {!}     if present requests write-back (W=1), otherwise W=0
+;;;   {^}     if present set S bit to load the CPSR along with the PC, or force transfer
+;;;           of user bank when in privileged mode
+;;; not to mention the alternative mnemonics PUSH and POP
+;;; when using the native stack pointer as base register.
+(define-instruction-format
+    ;; This is just to show something in the disassembly other than BYTE ...
+    (ldm/stm 32 :default-printer '(:name cond :tab bits ", " rn ", " reglist))
+  (cond :field (byte 4 28) :type 'condition-code)
+  (opcode-3 :field (byte 3 25))
+  (bits :field (byte 4 21)) ; complicated
+  (opcode-l :field (byte 1 20))
+  (rn :field (byte 4 16) :type 'reg)
+  (reglist :field (byte 16 0)
+           :printer (lambda (value stream dstate)
+                      (declare (ignore dstate))
+                      (format stream "{~{R~d~^,~}}"
+                              (loop for i below 16
+                                    when (logbitp i value) collect i)))))
+
 (define-instruction-format (swi 32
                             :default-printer '(:name cond :tab "#" swi-number))
   (cond :field (byte 4 28) :type 'condition-code)
@@ -1050,6 +1077,11 @@
   (define-load/store-instruction str :store :word)
   (define-load/store-instruction strb :store :byte))
 
+(define-instruction ldm (segment &rest args) ; load multiple
+  (:printer ldm/stm ((opcode-3 #b100) (opcode-l 1))))
+(define-instruction stm (segment &rest args) ; store multiple
+  (:printer ldm/stm ((opcode-3 #b100) (opcode-l 0))))
+
 ;;; Emit a miscellaneous load/store instruction.  CONDITION is a
 ;;; condition code name, OPCODE1 is the low bit of the first opcode
 ;;; field, OPCODE2 is the second opcode field, DATA is a register TN
@@ -1259,7 +1291,7 @@
      ;; apply the final 12 bits with LDR.  For now, we'll allow up to 20
      ;; bits of displacement, as that should be easy to implement, and a
      ;; megabyte large code object is already a bit unwieldly.  If
-     ;; neccessary, we can expand to a 28 bit displacement.
+     ;; necessary, we can expand to a 28 bit displacement.
      (labels ((compute-delta (position &optional magic-value)
                 (- (label-position label
                                    (when magic-value position)
