@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (declaim (special *compiler-error-bailout*))
 
@@ -21,6 +21,11 @@
 (declaim (type index *current-form-number*))
 (defvar *current-form-number*)
 
+;;; Report as we try each transform? (either source or IR)
+;;; Bind to T to see the ones that didn't abort,
+;;; or :ALL if you want to see each attempted transform.
+(defvar *show-transforms-p* nil)
+
 (declaim (inline source-form-has-path-p))
 (defun source-form-has-path-p (form)
   (not (typep form '(or symbol fixnum character))))
@@ -84,8 +89,6 @@
   to optimize code which uses those definitions? Setting this true
   gives non-ANSI, early-CMU-CL behavior. It can be useful for improving
   the efficiency of stable code.")
-
-(defvar *post-binding-variable-lexenv* nil)
 
 ;;;; namespace management utilities
 
@@ -96,22 +99,22 @@
   (let ((answer
          (typecase env
            (null nil)
-           #!+(and sb-fasteval (host-feature sb-xc))
-           (sb!interpreter:basic-env
-            (sb!interpreter::fun-lexically-notinline-p name env))
+           #+(and sb-fasteval (not sb-xc-host))
+           (sb-interpreter:basic-env
+            (sb-interpreter::fun-lexically-notinline-p name env))
            (t
             (let ((fun (cdr (assoc name (lexenv-funs env) :test #'equal))))
               ;; FIXME: this seems to omit FUNCTIONAL
               (when (defined-fun-p fun)
                 (return-from fun-lexically-notinline-p
-                  (eq (defined-fun-inlinep fun) :notinline))))))))
+                  (eq (defined-fun-inlinep fun) 'notinline))))))))
     ;; If ANSWER is NIL, go for the global value
-    (eq (or answer (info :function :inlinep name))
-        :notinline)))
+    (eq (or answer (info :function :inlinep name)) 'notinline)))
 
 (defun maybe-defined-here (name where)
   (if (and (eq :defined where)
-           (member name *fun-names-in-this-file* :test #'equal))
+           (boundp '*compilation*)
+           (member name (fun-names-in-this-file *compilation*) :test #'equal))
       :defined-here
       where))
 
@@ -124,7 +127,7 @@
   (let ((where (info :function :where-from name)))
     (when (and (eq where :assumed)
                ;; Slot accessors are defined just-in-time, if not already.
-               (not (typep name '(cons (eql sb!pcl::slot-accessor))))
+               (not (typep name '(cons (eql sb-pcl::slot-accessor))))
                ;; In the ordinary target Lisp, it's silly to report
                ;; undefinedness when the function is defined in the
                ;; running Lisp. But at cross-compile time, the current
@@ -137,7 +140,7 @@
                ;; complain about undefined functions.
                (not latep))
       (note-undefined-reference name :function))
-    (let ((ftype (proclaimed-ftype name))
+    (let ((ftype (global-ftype name))
           (notinline (fun-lexically-notinline-p name)))
       (make-global-var
        :kind :global-function
@@ -155,10 +158,10 @@
                        where
                        (maybe-defined-here name where))))))
 
-;;; Have some DEFINED-FUN-FUNCTIONALS of a *FREE-FUNS* entry become invalid?
+;;; Have some DEFINED-FUN-FUNCTIONALS of a FREE-FUNS entry become invalid?
 ;;; Drop 'em.
 ;;;
-;;; This was added to fix bug 138 in SBCL. It is possible for a *FREE-FUNS*
+;;; This was added to fix bug 138 in SBCL. It is possible for a FREE-FUNS
 ;;; entry to contain a DEFINED-FUN whose DEFINED-FUN-FUNCTIONAL object
 ;;; contained IR1 stuff (NODEs, BLOCKs...) referring to an already compiled
 ;;; (aka "dead") component. When this IR1 stuff was reused in a new component,
@@ -172,7 +175,7 @@
 ;;; BUGS entry also makes it seem like this might not be an issue at all on
 ;;; target.
 (defun clear-invalid-functionals (free-fun)
-  ;; There might be other reasons that *FREE-FUN* entries could
+  ;; There might be other reasons that FREE-FUN entries could
   ;; become invalid, but the only one we've been bitten by so far
   ;; (sbcl-0.pre7.118) is this one:
   (when (defined-fun-p free-fun)
@@ -201,15 +204,15 @@
                      (defined-fun-functionals free-fun)))
     nil))
 
-;;; If NAME already has a valid entry in *FREE-FUNS*, then return
+;;; If NAME already has a valid entry in (FREE-FUNS *IR1-NAMESPACE*), then return
 ;;; the value. Otherwise, make a new GLOBAL-VAR using information from
-;;; the global environment and enter it in *FREE-FUNS*. If NAME
+;;; the global environment and enter it in FREE-FUNS. If NAME
 ;;; names a macro or special form, then we error out using the
 ;;; supplied context which indicates what we were trying to do that
 ;;; demanded a function.
 (declaim (ftype (sfunction (t string) global-var) find-free-fun))
-(defun find-free-fun (name context)
-  (or (let ((old-free-fun (gethash name *free-funs*)))
+(defun find-free-fun (name context &aux (free-funs (free-funs *ir1-namespace*)))
+  (or (let ((old-free-fun (gethash name free-funs)))
         (when old-free-fun
           (clear-invalid-functionals old-free-fun)
           old-free-fun))
@@ -222,20 +225,20 @@
            (check-fun-name name)
            (let ((expansion (fun-name-inline-expansion name))
                  (inlinep (info :function :inlinep name)))
-             (setf (gethash name *free-funs*)
+             (setf (gethash name free-funs)
                    (if (or expansion inlinep)
                        (let ((where (info :function :where-from name)))
                          (make-defined-fun
                           :%source-name name
                           :inline-expansion expansion
                           :inlinep inlinep
-                          :where-from (if (eq inlinep :notinline)
+                          :where-from (if (eq inlinep 'notinline)
                                           where
                                           (maybe-defined-here name where))
-                          :type (if (and (eq inlinep :notinline)
+                          :type (if (and (eq inlinep 'notinline)
                                          (neq where :declared))
                                     (specifier-type 'function)
-                                    (proclaimed-ftype name))))
+                                    (global-ftype name))))
                        (find-global-fun name nil)))))))))
 
 ;;; Return the LEAF structure for the lexically apparent function
@@ -252,17 +255,18 @@
            (find-free-fun name context)))))
 
 (defun maybe-find-free-var (name)
-  (let ((found (gethash name *free-vars*)))
+  (let ((found (gethash name (free-vars *ir1-namespace*))))
     (unless (eq found :deprecated)
       found)))
 
 ;;; Return the LEAF node for a global variable reference to NAME. If
-;;; NAME is already entered in *FREE-VARS*, then we just return the
+;;; NAME is already entered in (FREE-VARS *IR1-NAMESPACE*), then we just return the
 ;;; corresponding value. Otherwise, we make a new leaf using
 ;;; information from the global environment and enter it in
-;;; *FREE-VARS*. If the variable is unknown, then we emit a warning.
+;;; FREE-VARS. If the variable is unknown, then we emit a warning.
 (declaim (ftype (sfunction (t) (or leaf cons heap-alien-info)) find-free-var))
-(defun find-free-var (name &aux (existing (gethash name *free-vars*)))
+(defun find-free-var (name &aux (free-vars (free-vars *ir1-namespace*))
+                                (existing (gethash name free-vars)))
   (unless (symbolp name)
     (compiler-error "Variable name is not a symbol: ~S." name))
   (or (when (and existing (neq existing :deprecated))
@@ -280,12 +284,12 @@
           (case deprecation-state
             ((:early :late)
              (check-deprecated-thing 'variable name))))
-        (setf (gethash name *free-vars*)
+        (setf (gethash name free-vars)
               (case kind
                 (:alien
                  (info :variable :alien-info name))
                 ;; FIXME: The return value in this case should really be
-                ;; of type SB!C::LEAF.  I don't feel too badly about it,
+                ;; of type SB-C::LEAF.  I don't feel too badly about it,
                 ;; because the MACRO idiom is scattered throughout this
                 ;; file, but it should be cleaned up so we're not
                 ;; throwing random conses around.  --njf 2002-03-23
@@ -294,20 +298,12 @@
                        (type (type-specifier (info :variable :type name))))
                    `(macro . (the ,type ,expansion))))
                 (:constant
-                 (let ((value (symbol-value name)))
-                   ;; Override the values of standard symbols in XC,
-                   ;; since we can't redefine them.
-                   #+sb-xc-host
-                   (when (eql (find-symbol (symbol-name name) :cl) name)
-                     (multiple-value-bind (xc-value foundp)
-                         (xc-constant-value name)
-                       (cond (foundp
-                              (setf value xc-value))
-                             ((not (eq value name))
-                              (compiler-warn
-                               "Using cross-compilation host's definition of ~S: ~A~%"
-                               name (symbol-value name))))))
-                   (find-constant value name)))
+                 #+sb-xc-host ; check that we never reference host constants
+                 (unless (member name '(nil t)) ; other than these 2
+                   (when (eq (find-symbol (string name) "XC-STRICT-CL")
+                             name)
+                     (error "Using a constant from the host ~s" name)))
+                 (find-constant (symbol-value name) name))
                 (t
                  (make-global-var :kind kind
                                   :%source-name name
@@ -320,21 +316,10 @@
 ;;; any subparts) are dumpable at all.
 (defun maybe-emit-make-load-forms (constant &optional (name nil namep))
   (let ((xset (alloc-xset)))
-    (labels ((trivialp (value)
-               (typep value
-                      '(or
-                        #-sb-xc-host
-                        (or unboxed-array #!+sb-simd-pack simd-pack)
-                        #+sb-xc-host
-                        (and array (not (array t)))
-                        symbol
-                        number
-                        character
-                        string))) ; subsumed by UNBOXED-ARRAY
-             (grovel (value)
+    (labels ((grovel (value)
                ;; Unless VALUE is an object which which obviously
                ;; can't contain other objects
-               (unless (trivialp value)
+               (unless (dumpable-leaflike-p value)
                  (if (xset-member-p value xset)
                      (return-from grovel nil)
                      (add-to-xset value xset))
@@ -358,8 +343,7 @@
                    ((array t)
                     (dotimes (i (array-total-size value))
                       (grovel (row-major-aref value i))))
-                   (#+sb-xc-host structure!object
-                    #-sb-xc-host instance
+                   (instance
                     ;; In the target SBCL, we can dump any instance, but
                     ;; in the cross-compilation host, %INSTANCE-FOO
                     ;; functions don't work on general instances, only on
@@ -374,21 +358,17 @@
                     (when (emit-make-load-form value)
                       #+sb-xc-host
                       (aver (eql (layout-bitmap (%instance-layout value))
-                                 sb!kernel::+layout-all-tagged+))
+                                 sb-kernel:+layout-all-tagged+))
                       (do-instance-tagged-slot (i value)
                         (grovel (%instance-ref value i)))))
                    (t
                     (compiler-error
-                      "Objects of type ~/sb!impl:print-type-specifier/ ~
+                      "Objects of type ~/sb-impl:print-type-specifier/ ~
                        can't be dumped into fasl files."
                      (type-of value)))))))
       ;; Dump all non-trivial named constants using the name.
-      (if (and namep (not (typep constant '(or symbol character
-                                            ;; FIXME: Cold init breaks if we
-                                            ;; try to reference FP constants
-                                            ;; thru their names.
-                                            #+sb-xc-host number
-                                            #-sb-xc-host fixnum))))
+      (if (and namep (not (sb-xc:typep constant '(or symbol character fixnum
+                                                  #+64-bit single-float))))
           (emit-make-load-form constant name)
           (grovel constant))))
   (values))
@@ -407,7 +387,7 @@
 ;;; determine what is evaluated next. If the ctran has no block, then
 ;;; we make it be in the block that the node is in. If the ctran heads
 ;;; its block, we end our block and link it to that block.
-#!-sb-fluid (declaim (inline use-ctran))
+#-sb-fluid (declaim (inline use-ctran))
 (defun use-ctran (node ctran)
   (declare (type node node) (type ctran ctran))
   (if (eq (ctran-kind ctran) :unused)
@@ -443,6 +423,15 @@
     (link-node-to-previous-ctran old temp))
   (values))
 
+(defun insert-node-before-no-split (old new)
+  (let ((prev (node-prev old))
+        (temp (make-ctran)))
+    (setf (ctran-next prev) nil)
+    (link-node-to-previous-ctran new prev)
+    (use-ctran new temp)
+    (link-node-to-previous-ctran old temp))
+  (values))
+
 ;;; This function is used to set the ctran for a node, and thus
 ;;; determine what receives the value.
 (defun use-lvar (node lvar)
@@ -460,7 +449,7 @@
            (setf (lvar-uses lvar) (list node (lvar-uses lvar)))))
     (reoptimize-lvar lvar)))
 
-#!-sb-fluid(declaim (inline use-continuation))
+#-sb-fluid(declaim (inline use-continuation))
 (defun use-continuation (node ctran lvar)
   (use-ctran node ctran)
   (use-lvar node lvar))
@@ -618,39 +607,29 @@
     (values)))
 
 ;;; Add FUNCTIONAL to the COMPONENT-REANALYZE-FUNCTIONALS, unless it's
-;;; some trivial type for which reanalysis is a trivial no-op, or
-;;; unless it doesn't belong in this component at all.
+;;; some trivial type for which reanalysis is a trivial no-op.
 ;;;
 ;;; FUNCTIONAL is returned.
 (defun maybe-reanalyze-functional (functional)
-
   (aver (not (eql (functional-kind functional) :deleted))) ; bug 148
   (aver-live-component *current-component*)
-
   ;; When FUNCTIONAL is of a type for which reanalysis isn't a trivial
   ;; no-op
   (when (typep functional '(or optional-dispatch clambda))
-
-    ;; When FUNCTIONAL knows its component
-    (when (lambda-p functional)
-      (aver (eql (lambda-component functional) *current-component*)))
-
     (pushnew functional
              (component-reanalyze-functionals *current-component*)))
-
   functional)
 
 ;;; Generate a REF node for LEAF, frobbing the LEAF structure as
 ;;; needed. If LEAF represents a defined function which has already
-;;; been converted, and is not :NOTINLINE, then reference the
+;;; been converted, and is not NOTINLINE, then reference the
 ;;; functional instead.
 (defun reference-leaf (start next result leaf &optional (name '.anonymous.))
   (declare (type ctran start next) (type (or lvar null) result) (type leaf leaf))
   (assure-leaf-live-p leaf)
   (let* ((type (lexenv-find leaf type-restrictions))
          (leaf (or (and (defined-fun-p leaf)
-                        (not (eq (defined-fun-inlinep leaf)
-                                 :notinline))
+                        (not (eq (defined-fun-inlinep leaf) 'notinline))
                         (let ((functional (defined-fun-functional leaf)))
                           (when (and functional (not (functional-kind functional)))
                             (maybe-reanalyze-functional functional))))
@@ -734,17 +713,17 @@
   (flet ((legal-cm-name-p (name)
            (and (legal-fun-name-p name)
                 (or (not (symbolp name))
-                    (not (sb!xc:macro-function name *lexenv*))))))
+                    (not (sb-xc:macro-function name *lexenv*))))))
     (if (eq opname 'funcall)
         (let ((fun-form (cadr form)))
           (cond ((and (consp fun-form) (eq 'function (car fun-form))
                       (not (cddr fun-form)))
                  (let ((real-fun (cadr fun-form)))
                    (if (legal-cm-name-p real-fun)
-                       (values (sb!xc:compiler-macro-function real-fun *lexenv*)
+                       (values (sb-xc:compiler-macro-function real-fun *lexenv*)
                                real-fun)
                        (values nil nil))))
-                ((sb!xc:constantp fun-form *lexenv*)
+                ((sb-xc:constantp fun-form *lexenv*)
                  (let ((fun (constant-form-value fun-form *lexenv*)))
                    (if (legal-cm-name-p fun)
                        ;; CLHS tells us that local functions must shadow
@@ -756,12 +735,12 @@
                        ;; a list (function name)", that means that
                        ;; (funcall 'name) that gets here doesn't fit the
                        ;; definition.
-                       (values (sb!xc:compiler-macro-function fun nil) fun)
+                       (values (sb-xc:compiler-macro-function fun nil) fun)
                        (values nil nil))))
                 (t
                  (values nil nil))))
         (if (legal-fun-name-p opname)
-            (values (sb!xc:compiler-macro-function opname *lexenv*) opname)
+            (values (sb-xc:compiler-macro-function opname *lexenv*) opname)
             (values nil nil)))))
 
 ;;; If FORM has a usable compiler macro, use it; otherwise return FORM itself.
@@ -819,8 +798,24 @@
               (ir1-convert-srctran start next result lexical-def form))
              (t
               (aver (and (consp lexical-def) (eq (car lexical-def) 'macro)))
-              (ir1-convert start next result
-                           (careful-expand-macro (cdr lexical-def) form))))))
+              ;; Unlike inline expansion, macroexpansion is not optional,
+              ;; but we clearly can't recurse forever.
+              (let ((expansions (memq lexical-def *inline-expansions*)))
+                (if (<= (or (cadr expansions) 0) *inline-expansion-limit*)
+                    (let ((*inline-expansions*
+                            (if expansions
+                                (progn (incf (cadr expansions))
+                                       *inline-expansions*)
+                                (list* lexical-def 1 *inline-expansions*))))
+                      (ir1-convert start next result
+                                   (careful-expand-macro (cdr lexical-def) form)))
+                    (progn
+                      (compiler-warn "Recursion limit reached while expanding local macro ~
+~/sb-ext:print-symbol-with-prefix/" op)
+                      ;; Treat it as an global function, which is probably
+                      ;; what was intended. The expansion thus far could be wrong,
+                      ;; but that's not our problem.
+                      (ir1-convert-global-functoid start next result form op))))))))
         ((or (atom op) (not (eq (car op) 'lambda)))
          (compiler-error "illegal function call"))
         (t
@@ -872,38 +867,7 @@
                             #+sb-xc-host "during XC macroexpansion"))
                      form
                      '*break-on-signals*))))
-    (handler-bind (;; KLUDGE: CMU CL in its wisdom (version 2.4.6 for Debian
-                   ;; Linux, anyway) raises a CL:WARNING condition (not a
-                   ;; CL:STYLE-WARNING) for undefined symbols when converting
-                   ;; interpreted functions, causing COMPILE-FILE to think the
-                   ;; file has a real problem, causing COMPILE-FILE to return
-                   ;; FAILURE-P set (not just WARNINGS-P set). Since undefined
-                   ;; symbol warnings are often harmless forward references,
-                   ;; and since it'd be inordinately painful to try to
-                   ;; eliminate all such forward references, these warnings
-                   ;; are basically unavoidable. Thus, we need to coerce the
-                   ;; system to work through them, and this code does so, by
-                   ;; crudely suppressing all warnings in cross-compilation
-                   ;; macroexpansion. -- WHN 19990412
-                   #+(and cmu sb-xc-host)
-                   (warning (lambda (c)
-                              (compiler-notify
-                               "~@<~A~:@_~
-                                ~A~:@_~
-                                ~@<(KLUDGE: That was a non-STYLE WARNING. ~
-                                   Ordinarily that would cause compilation to ~
-                                   fail. However, since we're running under ~
-                                   CMU CL, and since CMU CL emits non-STYLE ~
-                                   warnings for safe, hard-to-fix things (e.g. ~
-                                   references to not-yet-defined functions) ~
-                                   we're going to have to ignore it and ~
-                                   proceed anyway. Hopefully we're not ~
-                                   ignoring anything  horrible here..)~:@>~:>"
-                               (wherestring)
-                               c)
-                              (muffle-warning)
-                              (bug "no MUFFLE-WARNING restart")))
-                   (error
+    (handler-bind ((error
                      (lambda (c)
                        (cond
                          (cmacro
@@ -944,24 +908,12 @@
 
 ;;;; code coverage
 
-;;; Used as the CDR of the code coverage instrumentation records
-;;; (instead of NIL) to ensure that any well-behaving user code will
-;;; not have constants EQUAL to that record. This avoids problems with
-;;; the records getting coalesced with non-record conses, which then
-;;; get mutated when the instrumentation runs. Note that it's
-;;; important for multiple records for the same location to be
-;;; coalesced. -- JES, 2008-01-02
-;;; Use of #. mandates :COMPILE-TOPLEVEL for several Lisps
-;;; even though for us it's immediately accessible to EVAL.
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defconstant +code-coverage-unmarked+ '%code-coverage-unmarked%))
-
 ;;; Check the policy for whether we should generate code coverage
 ;;; instrumentation. If not, just return the original START
 ;;; ctran. Otherwise insert code coverage instrumentation after
 ;;; START, and return the new ctran.
 (defun instrument-coverage (start mode form
-                            &aux (metadata *compiler-coverage-metadata*))
+                            &aux (metadata (coverage-metadata *compilation*)))
   ;; We don't actually use FORM for anything, it's just convenient to
   ;; have around when debugging the instrumentation.
   (declare (ignore form))
@@ -984,9 +936,13 @@
                                    (cons path +code-coverage-unmarked+)))
                   (next (make-ctran))
                   (*allow-instrumenting* nil))
+              #+(or x86-64 x86) (declare (ignore store)) ; eval'd for side-effect only
               (push (ctran-block start)
                     (gethash path (code-coverage-blocks metadata)))
               (ir1-convert start next nil
+                           #+(or x86-64 x86)
+                           `(%primitive mark-covered ',path)
+                           #-(or x86-64 x86)
                            `(locally
                                 (declare (optimize speed
                                                    (safety 0)
@@ -1013,26 +969,6 @@
           (when *current-path*
             (instrument-coverage start nil form))))
       start))
-
-(defun record-code-coverage (info cc)
-  (setf (gethash info *code-coverage-info*) cc))
-
-(defun clear-code-coverage ()
-  (clrhash *code-coverage-info*))
-
-(defun reset-code-coverage ()
-  (maphash (lambda (info cc)
-             (declare (ignore info))
-             (dolist (cc-entry cc)
-               (setf (cdr cc-entry) +code-coverage-unmarked+)))
-           *code-coverage-info*))
-
-(defun code-coverage-record-marked (record)
-  (aver (consp record))
-  (ecase (cdr record)
-    ((#.+code-coverage-unmarked+) nil)
-    ((t) t)))
-
 
 ;;;; converting combinations
 
@@ -1046,12 +982,12 @@
            ;; if the value is unused.
            (if (member symbol '(symeval sym-global-val))
                (not (constantp (second form)))
-               (not (member (symbol-package symbol)
+               (not (member (sb-xc:symbol-package symbol)
                             (load-time-value
                               ;; KLUDGE: packages we're not interested in
                               ;; stepping.
-                              (mapcar #'find-package '(sb!c sb!int sb!impl
-                                                       sb!kernel sb!pcl))
+                              (mapcar #'find-package '(sb-c sb-int sb-impl
+                                                       sb-kernel sb-pcl))
                              t))))))
     (and *allow-instrumenting*
          (policy *lexenv* (= insert-step-conditions 3))
@@ -1116,9 +1052,13 @@
         (setf (combination-args node) (arg-lvars))))
     node))
 
-;;; Convert a call to a global function. If not :NOTINLINE, then we do
+(defun show-transform (kind name new-form)
+  (let ((*print-right-margin* 100))
+    (format *trace-output* "~&xform (~a) ~S~% -> ~S~%"
+            kind name new-form)))
+;;; Convert a call to a global function. If not NOTINLINE, then we do
 ;;; source transforms and try out any inline expansion. If there is no
-;;; expansion, but is :INLINE, then give an efficiency note (unless a
+;;; expansion, but is INLINE, then give an efficiency note (unless a
 ;;; known function which will quite possibly be open-coded.) Next, we
 ;;; go to ok-combination conversion.
 (defun ir1-convert-srctran (start next result var form)
@@ -1127,28 +1067,22 @@
   (let ((name (leaf-source-name var))
         (inlinep (when (defined-fun-p var)
                    (defined-fun-inlinep var))))
-    (if (eq inlinep :notinline)
+    (if (eq inlinep 'notinline)
         (ir1-convert-combination start next result form var)
-        (let* ((transform (info :function :source-transform name)))
+        (let ((transform (info :function :source-transform name)))
           (if transform
               (multiple-value-bind (transformed pass)
                   (if (functionp transform)
                       (funcall transform form *lexenv*)
-                      (let ((result
-                             (if (eq (cdr transform) :predicate)
-                                 (and (singleton-p (cdr form))
-                                      `(%instance-typep
-                                        ,(cadr form)
-                                        ',(dd-name (car transform))))
-                                 (slot-access-transform
-                                  (if (consp name) :write :read)
-                                  (cdr form) transform))))
-                        (values result (null result))))
+                      (struct-fun-transform transform form name))
+                ;; Note that "pass" means fail. Gotta love it.
                 (cond (pass
                        (ir1-convert-maybe-predicate start next result form var))
                       (t
                        (unless (policy *lexenv* (zerop store-xref-data))
                          (record-call name (ctran-block start) *current-path*))
+                       (when *show-transforms-p*
+                         (show-transform "src" name transformed))
                        (ir1-convert start next result transformed))))
               (ir1-convert-maybe-predicate start next result form var))))))
 
@@ -1278,8 +1212,8 @@
                                     'type-warning
                                     :format-control
                                     "The type declarations ~
-                                  ~/sb!impl:print-type/ and ~
-                                  ~/sb!impl:print-type/ for ~
+                                  ~/sb-impl:print-type/ and ~
+                                  ~/sb-impl:print-type/ for ~
                                   ~S conflict."
                                     :format-arguments
                                     (list old-type type var-name))))
@@ -1349,9 +1283,14 @@
 
 ;;; Process a special declaration, returning a new LEXENV. A non-bound
 ;;; special declaration is instantiated by throwing a special variable
-;;; into the variables if BINDING-FORM-P is NIL, or otherwise into
-;;; *POST-BINDING-VARIABLE-LEXENV*.
-(defun process-special-decl (spec res vars binding-form-p context)
+;;; into the variables if POST-BINDING-LEXENV is NIL,
+;;; or by mutating POST-BINDING-LEXENV if non-nil.
+;;; Note that POST-BINDING-LEXENV is not actually a lexenv - it is only
+;;; the contents of the 'vars' slot of a LEXENV, and moreover, as supplied
+;;; to this function, it has a dummy cons in front which is later removed.
+;;; The dummy cons serves a double purpose - as an indicator that this decl
+;;; occurs in a LET/LET* form, and facilitating pass by reference of the list.
+(defun process-special-decl (spec res vars post-binding-lexenv context)
   (declare (list spec vars) (type lexenv res))
   (collect ((new-venv nil cons))
     (dolist (name (cdr spec))
@@ -1388,9 +1327,9 @@
           (null
            (unless (or (assoc name (new-venv) :test #'eq))
              (new-venv (cons name (specvar-for-binding name))))))))
-    (cond (binding-form-p
-           (setf *post-binding-variable-lexenv*
-                 (append (new-venv) *post-binding-variable-lexenv*))
+    (cond (post-binding-lexenv
+           (setf (cdr post-binding-lexenv)
+                 (append (new-venv) (cdr post-binding-lexenv)))
            res)
           ((new-venv)
            (make-lexenv :default res :vars (new-venv)))
@@ -1401,7 +1340,7 @@
 ;;; (and TYPE if notinline), plus type-restrictions from the lexenv.
 (defun make-new-inlinep (var inlinep local-type)
   (declare (type global-var var) (type inlinep inlinep))
-  (let* ((type (if (and (eq inlinep :notinline)
+  (let* ((type (if (and (eq inlinep 'notinline)
                         (not (eq (leaf-where-from var) :declared)))
                    (specifier-type 'function)
                    (leaf-type var)))
@@ -1424,7 +1363,7 @@
 ;;; Parse an inline/notinline declaration. If it's a local function we're
 ;;; defining, set its INLINEP. If a global function, add a new FENV entry.
 (defun process-inline-decl (spec res fvars)
-  (let ((sense (cdr (assoc (first spec) +inlinep-translations+ :test #'eq)))
+  (let ((sense (first spec))
         (new-fenv ()))
     (dolist (name (rest spec))
       (let ((fvar (find name fvars
@@ -1526,13 +1465,11 @@
 (defun process-extent-decl (names vars fvars kind)
   (let ((extent
           (ecase kind
-            (truly-dynamic-extent
-             :always-dynamic)
             (dynamic-extent
              (when *stack-allocate-dynamic-extent*
-               :maybe-dynamic))
-            (indefinite-extent
-             :indefinite))))
+               kind))
+            ((indefinite-extent truly-dynamic-extent)
+             kind))))
     (if extent
         (dolist (name names)
           (cond
@@ -1560,7 +1497,7 @@
                   (eq (car name) 'function)
                   (null (cddr name))
                   (valid-function-name-p (cadr name))
-                  (neq :indefinite extent))
+                  (neq extent 'indefinite-extent))
              (let* ((fname (cadr name))
                     (bound-fun (find fname fvars
                                      :key (lambda (x)
@@ -1571,9 +1508,9 @@
                (etypecase fun
                  (leaf
                   (if bound-fun
-                      #!+stack-allocatable-closures
+                      #+stack-allocatable-closures
                       (setf (leaf-extent bound-fun) extent)
-                      #!-stack-allocatable-closures
+                      #-stack-allocatable-closures
                       (compiler-notify
                        "Ignoring DYNAMIC-EXTENT declaration on function ~S ~
                         (not supported on this platform)." fname)
@@ -1600,7 +1537,7 @@
 ;;; Process a single declaration spec, augmenting the specified LEXENV
 ;;; RES. Return RES and result type. VARS and FVARS are as described
 ;;; PROCESS-DECLS.
-(defun process-1-decl (raw-spec res vars fvars binding-form-p context)
+(defun process-1-decl (raw-spec res vars fvars post-binding-lexenv context)
   (declare (type list raw-spec vars fvars))
   (declare (type lexenv res))
   (let ((spec (canonized-decl-spec raw-spec))
@@ -1617,7 +1554,8 @@
        ((ignore ignorable)
         (process-ignore-decl spec vars fvars res)
         res)
-       (special (process-special-decl spec res vars binding-form-p context))
+       (special
+        (process-special-decl spec res vars post-binding-lexenv context))
        (ftype
         (unless (cdr spec)
           (compiler-error "no type specified in FTYPE declaration: ~S" spec))
@@ -1654,11 +1592,11 @@
        ;; We may want to detect LAMBDA-LIST and VALUES decls here,
        ;; and report them as "Misplaced" rather than "Unrecognized".
        (t
-        (unless (info :declaration :recognized (first spec))
-          (compiler-warn "unrecognized declaration ~S" raw-spec))
-        (let ((fn (info :declaration :handler (first spec))))
-          (if fn
-              (funcall fn res spec vars fvars)
+        (let ((info (info :declaration :known (first spec))))
+          (unless info
+            (compiler-warn "unrecognized declaration ~S" raw-spec))
+          (if (functionp info)
+              (funcall info res spec vars fvars)
               res))))
      optimize-qualities)))
 
@@ -1683,7 +1621,8 @@
         (allow-explicit-check allow-lambda-list)
         (lambda-list (if allow-lambda-list :unspecified nil))
         (optimize-qualities)
-        (*post-binding-variable-lexenv* nil))
+        source-form
+        (post-binding-lexenv (if binding-form-p (list nil)))) ; dummy cell
     (flet ((process-it (spec decl)
              (cond ((atom spec)
                     (compiler-error "malformed declaration specifier ~S in ~S"
@@ -1722,10 +1661,12 @@
                     (setq explicit-check (or (cdr spec) t)
                           allow-explicit-check nil)) ; at most one of this decl
                    ((equal spec '(top-level-form))) ; ignore
+                   ((typep spec '(cons (eql source-form)))
+                    (setf source-form (cadr spec)))
                    (t
                     (multiple-value-bind (new-env new-qualities)
                         (process-1-decl spec lexenv vars fvars
-                                        binding-form-p context)
+                                        post-binding-lexenv context)
                       (setq lexenv new-env
                             optimize-qualities
                             (nconc new-qualities optimize-qualities)))))))
@@ -1737,8 +1678,19 @@
               ;; Kludge: EVAL calls this function to deal with LOCALLY.
               (process-it spec decl)))))
     (warn-repeated-optimize-qualities (lexenv-policy lexenv) optimize-qualities)
-    (values lexenv result-type *post-binding-variable-lexenv*
-            lambda-list explicit-check)))
+    (values lexenv result-type (cdr post-binding-lexenv)
+            lambda-list explicit-check source-form)))
+
+(defun process-muffle-decls (decls lexenv)
+  (flet ((process-it (spec)
+           (cond ((atom spec))
+                 ((member (car spec) '(muffle-conditions unmuffle-conditions))
+                  (setq lexenv
+                        (process-1-decl spec lexenv nil nil nil nil))))))
+    (dolist (decl decls)
+      (dolist (spec (rest decl))
+        (process-it spec))))
+  lexenv)
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
   (multiple-value-bind (*lexenv* result-type post-binding-lexenv)
@@ -1759,10 +1711,9 @@
 (defmacro processing-decls ((decls vars fvars ctran lvar
                                    &optional post-binding-lexenv)
                             &body forms)
-  (check-type ctran symbol)
-  (check-type lvar symbol)
+  (declare (symbol ctran lvar))
   (let ((post-binding-lexenv-p (not (null post-binding-lexenv)))
-        (post-binding-lexenv (or post-binding-lexenv (sb!xc:gensym "LEXENV"))))
+        (post-binding-lexenv (or post-binding-lexenv (sb-xc:gensym "LEXENV"))))
     `(%processing-decls ,decls ,vars ,fvars ,ctran ,lvar
                         ,post-binding-lexenv-p
                         (lambda (,ctran ,lvar ,post-binding-lexenv)
