@@ -277,6 +277,7 @@
   (scs nil :type (or symbol list) :read-only t)
   ;; If non-null, we are a temp wired to this offset in SC.
   (offset nil :type (or unsigned-byte null) :read-only t))
+(declaim (freeze-type operand-parse))
 
 (defun operand-parse-sc (parse) ; Enforce a single symbol
   (the (and symbol (not null)) (operand-parse-scs parse)))
@@ -342,6 +343,7 @@
   (args-var '.args. :type symbol)
   (results-var '.results. :type symbol)
   (before-load :unspecified :type (or (member :unspecified) list)))
+(declaim (freeze-type vop-parse))
 (defprinter (vop-parse)
   name
   (inherits :test inherits)
@@ -1556,7 +1558,7 @@
 ;;; :MOVE-ARGS {NIL | :FULL-CALL | :LOCAL-CALL | :KNOWN-RETURN}
 ;;;     Indicates if and how the more args should be moved into a
 ;;;     different frame.
-(defmacro define-vop ((name &optional inherits) &body specs)
+(defmacro define-vop ((&optional name inherits) &body specs)
   (declare (type symbol name))
   ;; Parse the syntax into a VOP-PARSE structure, and then expand into
   ;; code that creates the appropriate VOP-INFO structure at load time.
@@ -1568,6 +1570,11 @@
                     (copy-vop-parse inherited-parse)
                     (make-vop-parse)))
          (n-res (gensym)))
+    (unless name
+      (let ((clause (assoc :translate specs)))
+        (when (singleton-p (cdr clause))
+          (setf name (cadr clause)))))
+    (aver (typep name '(and symbol (not null))))
     (setf (vop-parse-name parse) name)
     (setf (vop-parse-inherits parse) inherits)
 
