@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; Data object ref/set stuff.
 
@@ -142,19 +142,27 @@
   (:temporary (:scs (non-descriptor-reg)) type)
   (:results (result :scs (descriptor-reg)))
   (:generator 38
-    (let ((normal-fn (gen-label)))
       (load-type type function (- fun-pointer-lowtag))
       (inst nop)
       (inst xor type simple-fun-widetag)
       (inst beq type normal-fn)
       (inst addu lip function
-            (- (ash simple-fun-code-offset word-shift)
+            (- (ash simple-fun-insts-offset word-shift)
                fun-pointer-lowtag))
+      ;; XXX: it is questionable to load LIP with a value that is not an
+      ;; interior pointer relative to a boxed register. If there were any
+      ;; tagged pointer below LIP, then LIP might get "fixed" (i.e. ruined)
+      ;; by a GC if interrupted after this instruction. And it's even more
+      ;; questionable because 'li' is in fact two instructions.
+      ;; However, there are mitigating factors:
+      ;;   - there should be no boxed objects at a lower address.
+      ;;   - lack of thread support makes a GC interrupt somewhat unlikely,
+      ;;     (though any interrupt could call into lisp, also unlikely)
       (inst li lip (make-fixup 'closure-tramp :assembly-routine))
-      (emit-label normal-fn)
+    NORMAL-FN
       (storew lip fdefn fdefn-raw-addr-slot other-pointer-lowtag)
       (storew function fdefn fdefn-fun-slot other-pointer-lowtag)
-      (move result function))))
+      (move result function)))
 
 (define-vop (fdefn-makunbound)
   (:policy :fast-safe)
@@ -275,16 +283,15 @@
 
 ;;;; Instance hackery:
 
-(define-vop (instance-length)
+(define-vop ()
   (:policy :fast-safe)
   (:translate %instance-length)
   (:args (struct :scs (descriptor-reg)))
-  (:temporary (:scs (non-descriptor-reg)) temp)
   (:results (res :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 4
-    (loadw temp struct 0 instance-pointer-lowtag)
-    (inst srl res temp n-widetag-bits)))
+    (loadw res struct 0 instance-pointer-lowtag)
+    (inst srl res res n-widetag-bits)))
 
 (define-full-reffer instance-index-ref * instance-slots-offset
   instance-pointer-lowtag (descriptor-reg any-reg) * %instance-ref)
@@ -306,33 +313,37 @@
 
 ;;;; raw instance slot accessors
 
-(define-vop (raw-instance-ref/word)
-  (:translate %raw-instance-ref/word)
-  (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg))
-         (index :scs (any-reg)))
-  (:arg-types * positive-fixnum)
-  (:results (value :scs (unsigned-reg)))
-  (:temporary (:scs (interior-reg)) lip)
-  (:result-types unsigned-num)
-  (:generator 5
-    (inst addu lip object index)
-    (loadw value lip instance-slots-offset instance-pointer-lowtag)))
+(macrolet ((def (suffix sc primtype)
+             `(progn
+                (define-vop (,(symbolicate "RAW-INSTANCE-REF/" suffix))
+                  (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
+                  (:policy :fast-safe)
+                  (:args (object :scs (descriptor-reg))
+                         (index :scs (any-reg)))
+                  (:arg-types * positive-fixnum)
+                  (:results (value :scs (,sc)))
+                  (:temporary (:scs (interior-reg)) lip)
+                  (:result-types ,primtype)
+                  (:generator 5
+                    (inst addu lip object index)
+                    (loadw value lip instance-slots-offset instance-pointer-lowtag)))
 
-(define-vop (raw-instance-set/word)
-  (:translate %raw-instance-set/word)
-  (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg))
-         (index :scs (any-reg))
-         (value :scs (unsigned-reg) :target result))
-  (:arg-types * positive-fixnum unsigned-num)
-  (:results (result :scs (unsigned-reg)))
-  (:temporary (:scs (interior-reg)) lip)
-  (:result-types unsigned-num)
-  (:generator 5
-    (inst addu lip object index)
-    (storew value lip instance-slots-offset instance-pointer-lowtag)
-    (move result value)))
+                (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix))
+                  (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
+                  (:policy :fast-safe)
+                  (:args (object :scs (descriptor-reg))
+                         (index :scs (any-reg))
+                         (value :scs (,sc) :target result))
+                  (:arg-types * positive-fixnum ,primtype)
+                  (:results (result :scs (,sc)))
+                  (:temporary (:scs (interior-reg)) lip)
+                  (:result-types ,primtype)
+                  (:generator 5
+                    (inst addu lip object index)
+                    (storew value lip instance-slots-offset instance-pointer-lowtag)
+                    (move result value))))))
+  (def word unsigned-reg unsigned-num)
+  (def signed-word signed-reg signed-num))
 
 (define-vop (raw-instance-ref/single)
   (:translate %raw-instance-ref/single)
