@@ -115,7 +115,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     (let ((entry-point (gen-label)))
       (emit-label entry-point)
       ;; FIXME alpha port has a ### note here saying we should "save it
@@ -796,7 +796,7 @@ default-value-8
                   ;; is calculated.
                   (insert-step-instrumenting function)
                   (inst addi entry-point function
-                        (- (ash simple-fun-code-offset word-shift)
+                        (- (ash simple-fun-insts-offset word-shift)
                            fun-pointer-lowtag))))
                (:direct
                 `((inst lwz entry-point null-tn (static-fun-offset fun)))))
@@ -867,7 +867,7 @@ default-value-8
         (inst addi nsp-tn cur-nfp
               (- (bytes-needed-for-non-descriptor-stack-frame)
                  number-stack-displacement))))
-    (inst lr temp (make-fixup 'tail-call-variable :assembly-routine))
+    (load-asm-rtn-addr temp 'tail-call-variable)
     (inst mtlr temp)
     (inst blr)))
 
@@ -995,7 +995,7 @@ default-value-8
       (move old-fp old-fp-arg)
       (move vals vals-arg)
       (move nvals nvals-arg)
-      (inst lr temp (make-fixup 'return-multiple :assembly-routine))
+      (load-asm-rtn-addr temp 'return-multiple)
       (inst mtlr temp)
       (inst blr))))
 
@@ -1119,8 +1119,22 @@ default-value-8
   (:variant 0 0)
   (:translate %more-arg))
 
+(define-vop (more-arg-or-nil)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1))
+         (count :scs (any-reg)))
+  (:info index)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst cmpwi count (fixnumize index))
+    (move value null-tn)
+    (inst ble done)
+    (inst lwz value object (ash index word-shift))
+    done))
+
 ;;; Turn more arg (context, count) into a list.
-(define-vop (listify-rest-args)
+(define-vop ()
   (:args (context-arg :target context :scs (descriptor-reg))
          (count-arg :target count :scs (any-reg)))
   (:arg-types * tagged-num)
@@ -1158,7 +1172,7 @@ default-value-8
             (inst add csp-tn csp-tn temp))
           (progn
             (inst slwi temp count 1)
-            (allocation result temp list-pointer-lowtag
+            (allocation 'list temp list-pointer-lowtag result
                         :temp-tn dst
                         :flag-tn pa-flag)
             (move dst result)))
@@ -1196,7 +1210,7 @@ default-value-8
 ;;; preventing this info from being returned as values.  What we do is
 ;;; compute (- SUPPLIED FIXED), and return a pointer that many words
 ;;; below the current stack top.
-(define-vop (more-arg-context)
+(define-vop ()
   (:policy :fast-safe)
   (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg)))
