@@ -29,16 +29,17 @@
   (:results (result :scs (descriptor-reg) :from :eval))
   (:node-var node)
   (:generator 13
-    (inst lea bytes (ea (+ (* array-dimensions-offset n-word-bytes) lowtag-mask)
-                        nil rank (ash 1 (- word-shift n-fixnum-tag-bits))))
-    (inst and bytes (lognot lowtag-mask))
-    (inst lea header (ea (fixnumize (1- array-dimensions-offset)) rank))
-    (inst shl header n-widetag-bits)
-    (inst or  header type)
-    (inst shr header n-fixnum-tag-bits)
+    (inst lea :dword bytes
+          (ea (+ (* array-dimensions-offset n-word-bytes) lowtag-mask)
+              nil rank (ash 1 (- word-shift n-fixnum-tag-bits))))
+    (inst and :dword bytes (lognot lowtag-mask))
+    (inst lea :dword header (ea (fixnumize (1- array-dimensions-offset)) rank))
+    (inst shl :dword header n-widetag-bits)
+    (inst or  :dword header type)
+    (inst shr :dword header n-fixnum-tag-bits)
     (instrument-alloc bytes node)
     (pseudo-atomic ()
-     (allocation result bytes node nil 0)
+     (allocation nil bytes 0 node nil result)
      (storew header result 0 0)
      (inst or :byte result other-pointer-lowtag))))
 
@@ -58,7 +59,7 @@
                            type)))
      (instrument-alloc bytes node)
      (pseudo-atomic ()
-      (allocation result bytes node nil 0)
+      (allocation nil bytes 0 node nil result)
       (storew* header result 0 0 t)
       (inst or :byte result other-pointer-lowtag)))))
 
@@ -98,7 +99,7 @@
             (ea (- (* scale (1- array-dimensions-offset)))
                 (if (= scale 2) res nil) res (if (= scale 2) 1 scale))))))
 
-(define-vop (array-rank=)
+(define-vop ()
   (:translate %array-rank=)
   (:policy :fast-safe)
   (:args (array :scs (descriptor-reg)))
@@ -164,7 +165,9 @@
                     (inst cmp index bound)
                     (inst jmp :nb error))
                    (t
-                    (inst cmp bound index)
+                    (if (eql index 0)
+                        (inst test bound bound)
+                        (inst cmp bound index))
                     (inst jmp :be error))))))))
 (define-vop (check-bound/fast check-bound)
   (:policy :fast)
@@ -214,6 +217,20 @@
   (def-full-data-vector-frobs simple-array-unsigned-byte-63 unsigned-num
     unsigned-reg))
 
+(define-vop (data-vector-ref-with-offset/constant-simple-vector)
+  (:policy :fast-safe)
+  (:args (object :scs (constant))
+         (index :scs (any-reg)))
+  (:temporary (:sc unsigned-reg) array)
+  (:info offset)
+  (:arg-types simple-vector tagged-num (:constant fixnum))
+  (:results (value :scs (descriptor-reg any-reg immediate)))
+  (:result-types *)
+  (:generator 3
+    (inst lea array object)
+    (inst mov value (ea (* offset n-word-bytes) array
+                        index (ash 1 (- word-shift n-fixnum-tag-bits))))))
+
 (define-full-compare-and-swap %compare-and-swap-svref simple-vector
   vector-data-offset other-pointer-lowtag
   (descriptor-reg any-reg) *
@@ -419,7 +436,7 @@
   (def-small-data-vector-frobs simple-array-unsigned-byte-4 4))
 ;;; And the float variants.
 
-(defun make-ea-for-float-ref (object index offset element-size
+(defun float-ref-ea (object index offset element-size
                               &key (scale 1) (complex-offset 0))
   (etypecase index
       (integer
@@ -454,8 +471,8 @@
       ,@(if use-temp
             '((move dword-index index)
               (inst shr dword-index (1+ (- n-fixnum-tag-bits word-shift)))
-              (inst movss value (make-ea-for-float-ref object dword-index offset 4)))
-            '((inst movss value (make-ea-for-float-ref object index offset 4
+              (inst movss value (float-ref-ea object dword-index offset 4)))
+            '((inst movss value (float-ref-ea object index offset 4
                                  :scale (ash 4 (- n-fixnum-tag-bits)))))))))
 
 (define-vop (data-vector-ref-with-offset/simple-array-single-float-c)
@@ -470,7 +487,7 @@
   (:results (value :scs (single-reg)))
   (:result-types single-float)
   (:generator 4
-   (inst movss value (make-ea-for-float-ref object index offset 4))))
+   (inst movss value (float-ref-ea object index offset 4))))
 
 #.
 (let ((use-temp (<= word-shift n-fixnum-tag-bits)))
@@ -493,8 +510,8 @@
       ,@(if use-temp
             '((move dword-index index)
               (inst shr dword-index (1+ (- n-fixnum-tag-bits word-shift)))
-              (inst movss (make-ea-for-float-ref object dword-index offset 4) value))
-            '((inst movss (make-ea-for-float-ref object index offset 4
+              (inst movss (float-ref-ea object dword-index offset 4) value))
+            '((inst movss (float-ref-ea object index offset 4
                            :scale (ash 4 (- n-fixnum-tag-bits))) value)))
       (move result value))))
 
@@ -512,7 +529,7 @@
   (:results (result :scs (single-reg)))
   (:result-types single-float)
   (:generator 4
-   (inst movss (make-ea-for-float-ref object index offset 4) value)
+   (inst movss (float-ref-ea object index offset 4) value)
    (move result value)))
 
 (define-vop (data-vector-ref-with-offset/simple-array-double-float)
@@ -528,7 +545,7 @@
   (:results (value :scs (double-reg)))
   (:result-types double-float)
   (:generator 7
-   (inst movsd value (make-ea-for-float-ref object index offset 8
+   (inst movsd value (float-ref-ea object index offset 8
                                             :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
 
 (define-vop (data-vector-ref-c/simple-array-double-float)
@@ -543,7 +560,7 @@
   (:results (value :scs (double-reg)))
   (:result-types double-float)
   (:generator 6
-   (inst movsd value (make-ea-for-float-ref object index offset 8))))
+   (inst movsd value (float-ref-ea object index offset 8))))
 
 (define-vop (data-vector-set-with-offset/simple-array-double-float)
   (:note "inline array store")
@@ -560,7 +577,7 @@
   (:results (result :scs (double-reg)))
   (:result-types double-float)
   (:generator 20
-   (inst movsd (make-ea-for-float-ref object index offset 8
+   (inst movsd (float-ref-ea object index offset 8
                                       :scale (ash 1 (- word-shift n-fixnum-tag-bits)))
          value)
    (move result value)))
@@ -579,7 +596,7 @@
   (:results (result :scs (double-reg)))
   (:result-types double-float)
   (:generator 19
-   (inst movsd (make-ea-for-float-ref object index offset 8) value)
+   (inst movsd (float-ref-ea object index offset 8) value)
    (move result value)))
 
 
@@ -598,7 +615,7 @@
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 5
-    (inst movq value (make-ea-for-float-ref object index offset 8
+    (inst movq value (float-ref-ea object index offset 8
                                             :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
 
 (define-vop (data-vector-ref-with-offset/simple-array-complex-single-float-c)
@@ -613,7 +630,7 @@
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 4
-    (inst movq value (make-ea-for-float-ref object index offset 8))))
+    (inst movq value (float-ref-ea object index offset 8))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-single-float)
   (:note "inline array store")
@@ -631,7 +648,7 @@
   (:result-types complex-single-float)
   (:generator 5
     (move result value)
-    (inst movq (make-ea-for-float-ref object index offset 8
+    (inst movq (float-ref-ea object index offset 8
                                       :scale (ash 1 (- word-shift n-fixnum-tag-bits)))
           value)))
 
@@ -650,7 +667,7 @@
   (:result-types complex-single-float)
   (:generator 4
     (move result value)
-    (inst movq (make-ea-for-float-ref object index offset 8) value)))
+    (inst movq (float-ref-ea object index offset 8) value)))
 
 (define-vop (data-vector-ref-with-offset/simple-array-complex-double-float)
   (:note "inline array access")
@@ -665,7 +682,7 @@
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 7
-    (inst movapd value (make-ea-for-float-ref object index offset 16
+    (inst movapd value (float-ref-ea object index offset 16
                                               :scale (ash 2 (- word-shift n-fixnum-tag-bits))))))
 
 (define-vop (data-vector-ref-with-offset/simple-array-complex-double-float-c)
@@ -680,7 +697,7 @@
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 6
-    (inst movapd value (make-ea-for-float-ref object index offset 16))))
+    (inst movapd value (float-ref-ea object index offset 16))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-double-float)
   (:note "inline array store")
@@ -697,7 +714,7 @@
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 20
-    (inst movapd (make-ea-for-float-ref object index offset 16
+    (inst movapd (float-ref-ea object index offset 16
                                         :scale (ash 2 (- word-shift n-fixnum-tag-bits)))
           value)
     (move result value)))
@@ -716,7 +733,7 @@
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 19
-    (inst movapd (make-ea-for-float-ref object index offset 16) value)
+    (inst movapd (float-ref-ea object index offset 16) value)
     (move result value)))
 
 
@@ -815,7 +832,7 @@
     positive-fixnum unsigned-reg signed-reg)
   (define-data-vector-frobs simple-array-signed-byte-32 movsx :dword
     tagged-num signed-reg)
-  #!+sb-unicode
+  #+sb-unicode
   (define-data-vector-frobs simple-character-string movzx :dword
     character character-reg))
 
