@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; data object ref/set stuff
 
@@ -30,7 +30,7 @@
   (:results)
   (:vop-var vop)
   (:generator 1
-    (cond ((emit-gc-barrier-store-p name)
+    (cond ((emit-code-page-write-barrier-p name)
            ;; Don't assume that the immediate is acceptable to 'push'
            ;; (only MOV can take a 64-bit immediate)
            (inst mov temp-reg-tn (encode-value-if-immediate value))
@@ -53,16 +53,17 @@
                   (inst push offset)
                   (inst push object)))
            (invoke-asm-routine 'call 'code-header-set vop))
+          ((equal name '(setf %funcallable-instance-fun))
+           (gen-cell-set (object-slot-ea object offset lowtag) value nil vop t))
           (t
-           (gen-cell-set (make-ea-for-object-slot object offset lowtag)
-                         value nil)))))
+           (gen-cell-set (object-slot-ea object offset lowtag) value nil)))))
 
 ;; INIT-SLOT has to know about the :COMPACT-INSTANCE-HEADER feature.
 (define-vop (init-slot set-slot)
   (:info name dx-p offset lowtag)
   (:generator 1
     (progn name dx-p)
-    (cond #!+compact-instance-header
+    (cond #+compact-instance-header
           ((and (eq name '%make-structure-instance) (eql offset :layout))
         ;; The layout is in the upper half of the header word.
            (inst mov :dword (ea (- 4 instance-pointer-lowtag) object)
@@ -102,7 +103,7 @@
     (gen-cell-set (cond ((sc-is object immediate)
                          (symbol-slot-ea (tn-value object) symbol-value-slot))
                         (t
-                         (make-ea-for-object-slot object symbol-value-slot
+                         (object-slot-ea object symbol-value-slot
                                                   other-pointer-lowtag)))
                   value nil)))
 
@@ -138,9 +139,9 @@
 
 (deftransform %compare-and-swap-symbol-value ((symbol old new)
                                               ((constant-arg symbol) t t))
-  (if (eq (info :variable :kind (sb!c::lvar-value symbol)) :global)
+  (if (eq (info :variable :kind (sb-c::lvar-value symbol)) :global)
       `(%cas-symbol-global-value symbol old new)
-      (sb!c::give-up-ir1-transform)))
+      (sb-c::give-up-ir1-transform)))
 
 (macrolet (;; Logic common to thread-aware SET and CAS. CELL is assigned
            ;; to the location that should be accessed to modify SYMBOL's
@@ -170,7 +171,7 @@
            (new :scs (descriptor-reg any-reg)))
     (:temporary (:sc descriptor-reg :offset rax-offset
                  :from (:argument 1) :to (:result 0)) rax)
-    #!+sb-thread
+    #+sb-thread
     (:temporary (:sc descriptor-reg :to (:result 0)) cell)
     (:results (result :scs (descriptor-reg any-reg)))
     (:policy :fast-safe)
@@ -183,10 +184,10 @@
     ;; tls-index=0, because that would succeed, assigning NEW to each
     ;; symbol in existence having otherwise no thread-local value.
       (let ((unbound (generate-error-code vop 'unbound-symbol-error symbol)))
-        #!+sb-thread (progn (compute-virtual-symbol)
+        #+sb-thread (progn (compute-virtual-symbol)
                             (move rax old)
                             (inst cmpxchg (symbol-value-slot-ea cell) new :lock))
-        #!-sb-thread (progn (move rax old)
+        #-sb-thread (progn (move rax old)
                             ;; is the :LOCK is necessary?
                             (inst cmpxchg (symbol-value-slot-ea symbol) new :lock))
         (inst cmp :dword rax unbound-marker-widetag)
@@ -211,7 +212,7 @@
             new :lock)
       (move result rax)))
 
-  #!+sb-thread
+  #+sb-thread
   (progn
     ;; TODO: SET could be shorter for any known wired-tls symbol.
     (define-vop (set)
@@ -228,16 +229,11 @@
     ;; This code is tested by 'codegen.impure.lisp'
     (defun emit-symeval (value symbol symbol-reg check-boundp vop)
       (let* ((known-symbol-p (sc-is symbol constant immediate))
-             (known-symbol (and known-symbol-p (tn-value symbol)))
-             (*lexenv* (sb!c::node-lexenv (sb!c::vop-node vop))))
+             (known-symbol (and known-symbol-p (tn-value symbol))))
         ;; In order from best to worst.
         (cond
-          ((or (symbol-always-has-tls-value-p known-symbol) ; e.g. *HANDLER-CLUSTERS*
-               ;; bound here, has a TLS value
-               (and (symbol-always-has-tls-index-p known-symbol)
-                    (let ((var (lexenv-find known-symbol vars)))
-                      (and (sb!c::global-var-p var)
-                           (eq (sb!c::leaf-where-from var) :assumed)))))
+          ((symbol-always-has-tls-value-p known-symbol)
+           (setq symbol-reg nil)
            (inst mov value (access-wired-tls-val known-symbol)))
           (t
            (cond
@@ -293,7 +289,7 @@
                                 (lambda ()
                                   (load-immediate vop symbol symbol-reg)))
                             vop 'unbound-symbol-error
-                            (if (and immediatep (not staticp))
+                            (if (or (not symbol-reg) (and immediatep (not staticp)))
                                 symbol
                                 symbol-reg))))
             RETRY))))
@@ -329,17 +325,17 @@
       (:policy :fast-safe)
       (:args (object :scs (descriptor-reg)))
       (:conditional :ne)
-      (:temporary (:sc dword-reg) temp)
+      (:temporary (:sc unsigned-reg) temp)
       (:generator 9
-        (inst mov temp (tls-index-of object))
-        (inst mov temp (thread-tls-ea temp))
-        (inst cmp temp no-tls-value-marker-widetag)
-        (inst cmov :e temp (symbol-value-slot-ea object))
-        (inst cmp temp unbound-marker-widetag))))
+        (inst mov :dword temp (tls-index-of object))
+        (inst mov :dword temp (thread-tls-ea temp))
+        (inst cmp :dword temp no-tls-value-marker-widetag)
+        (inst cmov :dword :e temp (symbol-value-slot-ea object))
+        (inst cmp :dword temp unbound-marker-widetag))))
 
 ) ; END OF MACROLET
 
-#!-sb-thread
+#-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
     (:translate symeval))
@@ -352,7 +348,7 @@
     (:args (symbol :scs (descriptor-reg)))
     (:conditional :ne)
     (:generator 9
-      (inst cmp :dword (make-ea-for-object-slot
+      (inst cmp :dword (object-slot-ea
                  symbol symbol-value-slot other-pointer-lowtag)
             unbound-marker-widetag))))
 
@@ -364,11 +360,46 @@
   (:result-types positive-fixnum)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
-    ;; cdr slot, so we have to zero the fixnum tag bit(s) to make sure
+    ;; car slot, so we have to zero the fixnum tag bit(s) to make sure
     ;; it is a fixnum.  The lowtag selection magic that is required to
     ;; ensure this is explained in the comment in objdef.lisp
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
     (inst and res (lognot fixnum-tag-mask))))
+
+(eval-when (:compile-toplevel)
+  ;; assumption: any object can be read 1 word past its base pointer
+  (assert (= sb-vm:symbol-hash-slot 1)))
+
+(define-vop (symbol-hash*)
+  (:policy :fast-safe)
+  (:translate symbol-hash*)
+  (:args (symbol :scs (descriptor-reg)))
+  (:info satisfies)
+  (:arg-types * (:constant (member symbolp non-null-symbol-p)))
+  (:results (res :scs (any-reg)))
+  (:result-types positive-fixnum)
+  (:generator 2
+    (loadw res symbol symbol-hash-slot other-pointer-lowtag)
+    (when (eq satisfies 'symbolp) ; mask to a fixnum
+      (inst and res (lognot fixnum-tag-mask)))))
+(define-vop (symbol-hash*-random) ; this vop needs a temp; the above doesn't
+  (:policy :fast-safe)
+  (:translate symbol-hash*)
+  (:args (object :scs (descriptor-reg)))
+  (:info satisfies)
+  ;; arg can not target the temp because they both have to be live
+  ;; in order that the tagged pointer not disappear.
+  ;; But temp and output could be in the same register.
+  (:temporary (:sc unsigned-reg :to (:result 0)) base-ptr)
+  (:arg-types * (:constant (eql nil)))
+  (:results (res :scs (any-reg)))
+  (:result-types positive-fixnum)
+  (:ignore satisfies)
+  (:generator 4
+    (inst mov base-ptr object)
+    (inst and base-ptr (lognot lowtag-mask))
+    (inst mov res (ea n-word-bytes base-ptr)) ; 1 word beyond the header
+    (inst and res (lognot fixnum-tag-mask))))
 
 ;;;; fdefinition (FDEFN) objects
 
@@ -391,7 +422,7 @@
       (inst jmp :e err-lab))
     RETRY))
 
-#!-immobile-code
+#-immobile-code
 (define-vop (set-fdefn-fun)
   (:policy :fast-safe)
   (:translate (setf fdefn-fun))
@@ -408,6 +439,35 @@
     (storew function fdefn fdefn-fun-slot other-pointer-lowtag)
     (storew raw fdefn fdefn-raw-addr-slot other-pointer-lowtag)
     (move result function)))
+#+immobile-code
+(progn
+(define-vop (set-fdefn-fun)
+  (:args (fdefn :scs (descriptor-reg))
+         (function :scs (descriptor-reg))
+         (raw-word :scs (unsigned-reg)))
+  (:vop-var vop)
+  (:generator 38
+    (pseudo-atomic ()
+      (inst push fdefn)
+      (invoke-asm-routine 'call 'touch-gc-card vop)
+      (inst mov (ea (- (ash fdefn-fun-slot word-shift) other-pointer-lowtag) fdefn)
+            function)
+      (inst mov (ea (- (ash fdefn-raw-addr-slot word-shift) other-pointer-lowtag) fdefn)
+            raw-word)
+      ;; Ensure that the header contains a JMP instruction, not INT3.
+      ;; This store is aligned
+      (inst mov :word (ea (- 2 other-pointer-lowtag) fdefn) #x25FF))))
+(define-vop (set-undefined-fdefn-fun)
+  ;; Do not set the raw-addr slot and do not change the header
+  (:args (fdefn :scs (descriptor-reg))
+         (function :scs (descriptor-reg)))
+  (:vop-var vop)
+  (:generator 38
+    (pseudo-atomic ()
+      (inst push fdefn)
+      (invoke-asm-routine 'call 'touch-gc-card vop)
+      (inst mov (ea (- (ash fdefn-fun-slot word-shift) other-pointer-lowtag) fdefn)
+            function)))))
 
 (define-vop (fdefn-makunbound)
   (:policy :fast-safe)
@@ -415,28 +475,22 @@
   (:args (fdefn :scs (descriptor-reg) :target result))
   (:results (result :scs (descriptor-reg)))
   (:vop-var vop)
-  #!+immobile-code (:temporary (:sc unsigned-reg) temp)
   (:generator 38
-    #!+immobile-code
-    (let ((tramp (make-fixup 'undefined-fdefn :assembly-routine)))
-     (if (sb!c::code-immobile-p vop)
-         (inst lea temp (ea tramp rip-tn))
-         (inst mov temp tramp))
-     ;; Compute displacement from the call site
-     (inst sub :dword temp fdefn)
-     (inst sub :dword temp
-           (+ (- other-pointer-lowtag) (ash fdefn-raw-addr-slot word-shift) 5))
-     ;; Compute the encoding of a "CALL rel32" instruction
-     (inst shl temp 8)
-     (inst or :byte temp #xE8)
-     ;; Store
-     (storew nil-value fdefn fdefn-fun-slot other-pointer-lowtag)
-     (storew temp fdefn fdefn-raw-addr-slot other-pointer-lowtag))
-    #!-immobile-code
-    (progn
-     (storew nil-value fdefn fdefn-fun-slot other-pointer-lowtag)
-     (storew (make-fixup 'undefined-tramp :assembly-routine)
-             fdefn fdefn-raw-addr-slot other-pointer-lowtag))
+    ;; Change the JMP instruction to INT3 so that a trap occurs in the fdefn
+    ;; itself, otherwise we've no way of knowing what function name was invoked.
+    (inst mov :word (ea (- 2 other-pointer-lowtag) fdefn)
+          (logand undefined-fdefn-header #xFFFF))
+    ;; Once the opcode is written, the values in 'fun' and 'raw-addr' become irrelevant.
+    ;; These stores act primarily to clear the reference from a GC perspective.
+    (storew nil-value fdefn fdefn-fun-slot other-pointer-lowtag)
+    ;; With #+immobile-code we never call via the raw-addr slot for undefined
+    ;; functions if the single instruction "call <fdefn>" form is used. The INT3
+    ;; raises sigtrap which we catch, then load RAX with the address of the fdefn
+    ;; and resume at undefined-tramp. However, CALL-SYMBOL jumps via raw-addr if
+    ;; its callable object was not a function. In that case RAX holds a symbol,
+    ;; so we're OK because we can identify the undefined function.
+    (storew (make-fixup 'undefined-tramp :assembly-routine)
+            fdefn fdefn-raw-addr-slot other-pointer-lowtag)
     (move result fdefn)))
 
 ;;;; binding and unbinding
@@ -446,7 +500,7 @@
 ;;; symbol.
 ;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
 
-#!+sb-thread
+#+sb-thread
 (progn
 (define-vop (dynbind) ; bind a symbol in a PROGV form
   (:args (val :scs (any-reg descriptor-reg))
@@ -491,7 +545,7 @@
       (inst mov :dword (ea (ash binding-symbol-slot word-shift) bsp) tls-index)
       (inst mov :qword tls-cell (encode-value-if-immediate val))))))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -505,8 +559,8 @@
     (storew symbol bsp (- binding-symbol-slot binding-size))
     (storew val symbol symbol-value-slot other-pointer-lowtag)))
 
-#!+sb-thread
-(define-vop (unbind)
+#+sb-thread
+(define-vop (unbind-n)
   (:temporary (:sc unsigned-reg) temp bsp)
   (:temporary (:sc complex-double-reg) zero)
   (:info symbols)
@@ -526,7 +580,7 @@
           (inst movapd (ea bsp) zero))
     (store-binding-stack-pointer bsp)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) symbol value bsp)
   (:generator 0
@@ -550,19 +604,19 @@
     (inst sub bsp (* binding-size n-word-bytes))
     ;; on sb-thread symbol is actually a tls-index, and it fits into
     ;; 32-bits.
-    #!+sb-thread
+    #+sb-thread
     (progn
       (inst mov :dword symbol (ea (* binding-symbol-slot n-word-bytes) bsp))
       (inst test :dword symbol symbol))
-    #!-sb-thread
+    #-sb-thread
     (progn
       (loadw symbol bsp binding-symbol-slot)
       (inst test symbol symbol))
     (inst jmp :z SKIP)
     (loadw value bsp binding-value-slot)
-    #!-sb-thread
+    #-sb-thread
     (storew value symbol symbol-value-slot other-pointer-lowtag)
-    #!+sb-thread
+    #+sb-thread
     (inst mov (thread-tls-ea symbol) value)
 
     SKIP
@@ -625,7 +679,7 @@
 
 ;;;; structure hackery
 
-(define-vop (instance-length)
+(define-vop ()
   (:policy :fast-safe)
   (:translate %instance-length)
   (:args (struct :scs (descriptor-reg)))
@@ -637,7 +691,7 @@
     (inst movzx '(:word :dword) res (ea (1+ (- instance-pointer-lowtag)) struct))
     (inst shl :dword res n-fixnum-tag-bits)))
 
-#!+compact-instance-header
+#+compact-instance-header
 (progn
  (define-vop (%instance-layout)
    (:translate %instance-layout)
@@ -656,8 +710,15 @@
    (:results (res :scs (any-reg descriptor-reg)))
    (:variant-vars lowtag)
    (:variant instance-pointer-lowtag)
-   (:generator 2
-    (inst mov :dword (ea (- 4 lowtag) object) value)
+   (:vop-var vop)
+   (:generator 2 ; underestimates cost, but irrelevant as there's no choice
+    (cond ((= lowtag fun-pointer-lowtag)
+           (pseudo-atomic ()
+             (inst push object)
+             (invoke-asm-routine 'call 'touch-gc-card vop)
+             (inst mov :dword (ea (- 4 lowtag) object) value)))
+          (t
+           (inst mov :dword (ea (- 4 lowtag) object) value)))
     (move res value)))
  (define-vop (%funcallable-instance-layout %instance-layout)
    (:translate %funcallable-instance-layout)
@@ -705,7 +766,7 @@
                      (- instance-pointer-lowtag))
                   object index (ash 1 (- word-shift n-fixnum-tag-bits)))))))
   (macrolet
-      ((def (suffix result-sc result-type inst &optional (inst/c inst))
+      ((def (suffix result-sc result-type inst &optional (inst/c (list 'quote inst)))
          `(progn
             (define-vop (,(symbolicate "RAW-INSTANCE-REF/" suffix))
               (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
@@ -729,7 +790,7 @@
               (:results (value :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 4
-                (inst ,inst/c value (instance-slot-ea object index))))
+                (inst* ,inst/c value (instance-slot-ea object index))))
             (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix))
               (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
               (:policy :fast-safe)
@@ -755,7 +816,7 @@
               (:results (result :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 4
-                (inst ,inst/c (instance-slot-ea object index) value)
+                (inst* ,inst/c (instance-slot-ea object index) value)
                 (move result value)))
             (define-vop (,(symbolicate "RAW-INSTANCE-INIT/" suffix))
               (:args (object :scs (descriptor-reg))
@@ -763,7 +824,7 @@
               (:arg-types * ,result-type)
               (:info index)
               (:generator 4
-                (inst ,inst/c (instance-slot-ea object index) value))))))
+                (inst* ,inst/c (instance-slot-ea object index) value))))))
     (def word unsigned-reg unsigned-num mov)
     (def signed-word signed-reg signed-num mov)
     (def single single-reg single-float movss)
