@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; moves and coercions
 
@@ -30,7 +30,7 @@
   (:note "SAP to pointer coercion")
   (:node-var node)
   (:generator 20
-    (fixed-alloc res sap-widetag sap-size node)
+    (alloc-other res sap-widetag sap-size node)
     (storew sap res sap-pointer-slot other-pointer-lowtag)))
 (define-move-vop move-from-sap :move
   (sap-reg) (descriptor-reg))
@@ -99,9 +99,9 @@
   (:generator 1
     (move sap int)))
 
-;;;; POINTER+ and POINTER-
+;;;; SAP+ and SAP-
 
-(define-vop (pointer+)
+(define-vop ()
   (:translate sap+)
   (:args (ptr :scs (sap-reg) :target res
               :load-if (not (location= ptr res)))
@@ -138,7 +138,7 @@
                        (inst mov temp value)
                        (inst add res temp))))))))))
 
-(define-vop (pointer-)
+(define-vop ()
   (:translate sap-)
   (:args (ptr1 :scs (sap-reg) :target res)
          (ptr2 :scs (sap-reg)))
@@ -154,21 +154,62 @@
 
 ;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
 ;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
-#!+linux ; shadow space differs by OS
+#+linux ; shadow space differs by OS
 (defconstant msan-mem-to-shadow-xor-const #x500000000000)
 
-(defun emit-sap-set (size sap ea-index ea-disp value result)
-  #!+linux
-  (when sb!c::*msan-unpoison*
-    (let ((offset (or ea-index ea-disp)))
-      (unless (eql offset 0)
-        (inst add sap offset))
-      (inst mov temp-reg-tn msan-mem-to-shadow-xor-const)
-      (inst xor temp-reg-tn sap)
-      (inst mov size (ea 0 temp-reg-tn) 0)
-      (unless (eql offset 0) ; restore SAP as if nothing happened
-        (inst sub sap offset))))
-  (inst mov size (ea ea-disp sap ea-index) value)
+#|
+https://llvm.org/doxygen/MemorySanitizer_8cpp.html
+/// "We load the shadow _after_ the application load,
+/// and we store the shadow _before_ the app store."
+|#
+
+(defun emit-sap-ref (size insn modifier result ea node vop)
+  (declare (ignorable node size vop))
+  (cond
+   #+linux
+   ((and (sb-c:msan-unpoison sb-c:*compilation*) (policy node (> safety 0)))
+    ;; Must not clobber TEMP-REG-TN with the load.
+    (aver (not (location= temp-reg-tn result)))
+    (inst lea temp-reg-tn ea)
+    (sb-assem:inst* insn modifier result (ea temp-reg-tn))
+    (inst xor temp-reg-tn (thread-slot-ea thread-msan-xor-constant-slot))
+    ;; Per the documentation, shadow is tested _after_
+    (let ((mask (sb-c::masked-memory-load-p vop))
+          (good (gen-label))
+          (nbytes (size-nbyte size))
+          bad)
+      ;; If the load is going to be masked, then we must only check the
+      ;; shadow bits under the mask.
+      (cond ((not mask)
+             (inst cmp size (ea temp-reg-tn) 0))
+            ((or (neq size :qword) (plausible-signed-imm32-operand-p mask))
+             (inst test size (ea temp-reg-tn)
+                   (ldb (byte (* 8 nbytes) 0) mask)))
+            (t
+             ;; Test two 32-bit chunks of the shadow memory since we don't
+             ;; have an available register to load a 64-bit constant.
+             (inst test :dword (ea temp-reg-tn) (ldb (byte 32 0) mask))
+             (setq bad (gen-label))
+             (inst jmp :ne bad)
+             (inst test :dword (ea 4 temp-reg-tn) (ldb (byte 32 32) mask))))
+      (inst jmp :e good)
+      (when bad (emit-label bad))
+      (inst break sb-vm:uninitialized-load-trap)
+      ;; Encode the target size and register. If XMM register loads were sanitized,
+      ;; then this would need some more bits to indicate the register file.
+      (let ((scale (1- (integer-length nbytes))))
+        (inst byte (logior (ash (tn-offset result) 2) scale)))
+      (emit-label good)))
+   (t
+    (sb-assem::inst* insn modifier result ea))))
+
+(defun emit-sap-set (size ea value result)
+  #+linux
+  (when (sb-c:msan-unpoison sb-c:*compilation*)
+    (inst lea temp-reg-tn ea)
+    (inst xor temp-reg-tn (thread-slot-ea thread-msan-xor-constant-slot))
+    (inst mov size (ea temp-reg-tn) 0))
+  (inst mov size ea value)
   (move result value))
 
 (macrolet ((def-system-ref-and-set (ref-name
@@ -189,19 +230,22 @@
                     (:arg-types system-area-pointer signed-num)
                     (:results (result :scs (,sc)))
                     (:result-types ,type)
-                    (:generator 5
-                      (inst ,ref-insn ',modifier result (ea sap offset))))
+                    (:node-var node)
+                    (:vop-var vop)
+                    (:generator 3 (emit-sap-ref ,size ',ref-insn
+                                                ',modifier result (ea sap offset) node vop)))
                   (define-vop (,ref-name-c)
                     (:translate ,ref-name)
                     (:policy :fast-safe)
                     (:args (sap :scs (sap-reg)))
-                    (:arg-types system-area-pointer
-                                (:constant (signed-byte 32)))
+                    (:arg-types system-area-pointer (:constant (signed-byte 32)))
                     (:info offset)
                     (:results (result :scs (,sc)))
                     (:result-types ,type)
-                    (:generator 4
-                      (inst ,ref-insn ',modifier result (ea offset sap))))
+                    (:node-var node)
+                    (:vop-var vop)
+                    (:generator 2 (emit-sap-ref ,size ',ref-insn
+                                                ',modifier result (ea offset sap) node vop)))
                   (define-vop (,set-name)
                     (:translate ,set-name)
                     (:policy :fast-safe)
@@ -211,18 +255,17 @@
                     (:arg-types system-area-pointer signed-num ,type)
                     (:results (result :scs (,sc)))
                     (:result-types ,type)
-                    (:generator 5 (emit-sap-set ,size sap offset 0 value result)))
+                    (:generator 5 (emit-sap-set ,size (ea sap offset) value result)))
                   (define-vop (,set-name-c)
                     (:translate ,set-name)
                     (:policy :fast-safe)
                     (:args (sap :scs (sap-reg) :to (:eval 0))
                            (value :scs (,sc) :target result))
-                    (:arg-types system-area-pointer
-                                (:constant (signed-byte 32)) ,type)
+                    (:arg-types system-area-pointer (:constant (signed-byte 32)) ,type)
                     (:info offset)
                     (:results (result :scs (,sc)))
                     (:result-types ,type)
-                    (:generator 4 (emit-sap-set ,size sap nil offset value result)))))))
+                    (:generator 4 (emit-sap-set ,size (ea offset sap) value result)))))))
 
   (def-system-ref-and-set sap-ref-8 %set-sap-ref-8 movzx
     unsigned-reg positive-fixnum :byte)
@@ -245,106 +288,51 @@
   (def-system-ref-and-set sap-ref-lispobj %set-sap-ref-lispobj mov
     descriptor-reg * :qword))
 
-;;;; SAP-REF-DOUBLE
-
-(define-vop (sap-ref-double)
-  (:translate sap-ref-double)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg))
-         (offset :scs (signed-reg)))
-  (:arg-types system-area-pointer signed-num)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
-  (:generator 5
-     (inst movsd result (ea sap offset))))
-
-(define-vop (sap-ref-double-c)
-  (:translate sap-ref-double)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg)))
-  (:arg-types system-area-pointer (:constant (signed-byte 32)))
-  (:info offset)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
-  (:generator 4
-     (inst movsd result (ea offset sap))))
-
-(define-vop (%set-sap-ref-double)
-  (:translate %set-sap-ref-double)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg) :to (:eval 0))
-         (offset :scs (signed-reg) :to (:eval 0))
-         (value :scs (double-reg)))
-  (:arg-types system-area-pointer signed-num double-float)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
-  (:generator 5
-    (inst movsd (ea sap offset) value)
-    (move result value)))
-
-(define-vop (%set-sap-ref-double-c)
-  (:translate %set-sap-ref-double)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg) :to (:eval 0))
-         (value :scs (double-reg)))
-  (:arg-types system-area-pointer (:constant (signed-byte 32)) double-float)
-  (:info offset)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
-  (:generator 4
-    (inst movsd (ea offset sap) value)
-    (move result value)))
-
-;;;; SAP-REF-SINGLE
-
-(define-vop (sap-ref-single)
-  (:translate sap-ref-single)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg))
-         (offset :scs (signed-reg)))
-  (:arg-types system-area-pointer signed-num)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
-  (:generator 5
-     (inst movss result (ea sap offset))))
-
-(define-vop (sap-ref-single-c)
-  (:translate sap-ref-single)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg)))
-  (:arg-types system-area-pointer (:constant (signed-byte 32)))
-  (:info offset)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
-  (:generator 4
-     (inst movss result (ea offset sap))))
-
-(define-vop (%set-sap-ref-single)
-  (:translate %set-sap-ref-single)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg) :to (:eval 0))
-         (offset :scs (signed-reg) :to (:eval 0))
-         (value :scs (single-reg)))
-  (:arg-types system-area-pointer signed-num single-float)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
-  (:generator 5
-    (inst movss (ea sap offset) value)
-    (move result value)))
-
-(define-vop (%set-sap-ref-single-c)
-  (:translate %set-sap-ref-single)
-  (:policy :fast-safe)
-  (:args (sap :scs (sap-reg) :to (:eval 0))
-         (value :scs (single-reg)))
-  (:arg-types system-area-pointer (:constant (signed-byte 32)) single-float)
-  (:info offset)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
-  (:generator 4
-    (inst movss (ea offset sap) value)
-    (move result value)))
+;;;; SAP-REF-SINGLE and SAP-REF-DOUBLE
 
+(macrolet ((def-system-ref-and-set (ref-fun res-sc res-type insn
+                                            &aux (set-fun (symbolicate "%SET-" ref-fun)))
+             `(progn
+                (define-vop (,ref-fun)
+                  (:translate ,ref-fun)
+                  (:policy :fast-safe)
+                  (:args (sap :scs (sap-reg))
+                         (offset :scs (signed-reg)))
+                  (:arg-types system-area-pointer signed-num)
+                  (:results (result :scs (,res-sc)))
+                  (:result-types ,res-type)
+                  (:generator 5 (inst ,insn result (ea sap offset))))
+                (define-vop (,(symbolicate ref-fun "-C"))
+                  (:translate ,ref-fun)
+                  (:policy :fast-safe)
+                  (:args (sap :scs (sap-reg)))
+                  (:arg-types system-area-pointer (:constant (signed-byte 32)))
+                  (:info offset)
+                  (:results (result :scs (,res-sc)))
+                  (:result-types ,res-type)
+                  (:generator 4 (inst ,insn result (ea offset sap))))
+                (define-vop (,set-fun)
+                  (:translate ,set-fun)
+                  (:policy :fast-safe)
+                  (:args (sap :scs (sap-reg) :to (:eval 0))
+                         (offset :scs (signed-reg) :to (:eval 0))
+                         (value :scs (,res-sc)))
+                  (:arg-types system-area-pointer signed-num ,res-type)
+                  (:results (result :scs (,res-sc)))
+                  (:result-types ,res-type)
+                  (:generator 5 (inst ,insn (ea sap offset) value) (move result value)))
+                (define-vop (,(symbolicate set-fun "-C"))
+                  (:translate ,set-fun)
+                  (:policy :fast-safe)
+                  (:args (sap :scs (sap-reg) :to (:eval 0))
+                         (value :scs (,res-sc)))
+                  (:arg-types system-area-pointer (:constant (signed-byte 32)) ,res-type)
+                  (:info offset)
+                  (:results (result :scs (,res-sc)))
+                  (:result-types ,res-type)
+                  (:generator 4 (inst ,insn (ea offset sap) value) (move result value))))))
+  (def-system-ref-and-set sap-ref-single single-reg single-float movss)
+  (def-system-ref-and-set sap-ref-double double-reg double-float movsd))
 
 ;;; noise to convert normal lisp data objects into SAPs
 
@@ -367,7 +355,7 @@
          (offset :scs (signed-reg) :to (:eval 0))
          (oldval :scs (signed-reg) :target eax)
          (newval :scs (signed-reg) :to (:eval 0)))
-  (:temporary (:sc dword-reg :offset eax-offset
+  (:temporary (:sc unsigned-reg :offset eax-offset
                    :from (:argument 2) :to (:result 0)) eax)
   (:arg-types system-area-pointer signed-num signed-num signed-num)
   (:results (result :scs (signed-reg)))
@@ -375,5 +363,5 @@
   (:generator 5
     (inst mov :dword eax oldval)
     (inst cmpxchg :dword (ea sap offset) newval :lock)
-    (inst mov :dword result eax)))
+    (inst movsx '(:dword :qword) result eax)))
 
