@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defconstant arg-count-sc (make-sc+offset any-reg-sc-number ecx-offset))
 (defconstant closure-sc (make-sc+offset descriptor-reg-sc-number eax-offset))
@@ -199,10 +199,10 @@
                                 (list (primitive-type-or-lose ',name)
                                       (sc-or-lose ',stack-sc)
                                       (lambda (node block fp value res)
-                                        (sb!c::vop ,ref node block
+                                        (sb-c::vop ,ref node block
                                                    fp value res))
                                       (lambda (node block fp new-val value)
-                                        (sb!c::vop ,set node block
+                                        (sb-c::vop ,set node block
                                                    fp new-val value)))))))))
     (foo (double-float double-stack
                        ancestor-frame-ref/double-float
@@ -232,11 +232,14 @@
 (define-vop (xep-allocate-frame)
   (:info start-lab)
   (:generator 1
-    (emit-alignment n-lowtag-bits)
+    (let ((nop-kind
+           (shiftf (sb-assem::asmstream-inter-function-padding sb-assem:*asmstream*)
+                   :nop)))
+      (emit-alignment n-lowtag-bits (if (eq nop-kind :nop) #x90 0)))
     (emit-label start-lab)
     ;; Skip space for the function header.
     (inst simple-fun-header-word)
-    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     ;; The start of the actual code.
     ;; Save the return-pc.
     (popw ebp-tn (frame-word-offset return-pc-save-offset))))
@@ -295,7 +298,7 @@
 (defun default-unknown-values (vop values nvals node)
   (declare (type (or tn-ref null) values)
            (type unsigned-byte nvals))
-  (let ((type (sb!c::basic-combination-derived-type node)))
+  (let ((type (sb-c::basic-combination-derived-type node)))
     (cond
       ((<= nvals 1)
        (note-this-location vop :single-value-return)
@@ -400,12 +403,12 @@
                    (inst mov (second default) eax-tn))
                  (inst jmp defaulting-done)))))))
       (t
-       ;; 91 bytes for this branch.
        (let ((regs-defaulted (gen-label))
              (restore-edi (gen-label))
              (no-stack-args (gen-label))
              (default-stack-vals (gen-label))
-             (count-okay (gen-label)))
+             (count-okay (gen-label))
+             (copy-loop (gen-label)))
          (note-this-location vop :unknown-return)
          ;; Branch off to the MV case.
          (inst jmp :c regs-defaulted)
@@ -424,8 +427,7 @@
          ;; Load EAX with NIL so we can quickly store it, and set up
          ;; stuff for the loop.
          (inst mov eax-tn nil-value)
-         (inst std)
-         (inst mov ecx-tn (- nvals register-arg-count))
+         (inst mov ecx-tn (fixnumize (- nvals register-arg-count)))
          ;; Jump into the default loop.
          (inst jmp default-stack-vals)
          ;; The regs are defaulted. We need to copy any stack arguments,
@@ -455,10 +457,15 @@
                         :disp (frame-byte-offset
                                (+ sp->fp-offset register-arg-count))))
          ;; Do the copy.
-         (inst shr ecx-tn word-shift)   ; make word count
-         (inst std)
-         (inst rep)
-         (inst movs :dword)
+         (inst push edx-tn)
+         (emit-label copy-loop)
+         (inst mov edx-tn (make-ea :dword :base esi-tn))
+         (inst sub esi-tn n-word-bytes)
+         (inst mov (make-ea :dword :base edi-tn) edx-tn)
+         (inst sub edi-tn n-word-bytes)
+         (inst sub ecx-tn (fixnumize 1))
+         (inst jmp :nz copy-loop)
+         (inst pop edx-tn)
          ;; Restore ESI.
          (loadw esi-tn ebx-tn (frame-word-offset (+ sp->fp-offset 2)))
          ;; Now we have to default the remaining args. Find out how many.
@@ -467,18 +474,18 @@
          ;; If none, then just blow out of here.
          (inst jmp :le restore-edi)
          (inst mov ecx-tn eax-tn)
-         (inst shr ecx-tn word-shift)   ; word count
          ;; Load EAX with NIL for fast storing.
          (inst mov eax-tn nil-value)
          ;; Do the store.
          (emit-label default-stack-vals)
-         (inst rep)
-         (inst stos eax-tn)
+         (inst mov (make-ea :dword :base edi-tn) eax-tn)
+         (inst sub edi-tn n-word-bytes)
+         (inst sub ecx-tn (fixnumize 1))
+         (inst jmp :nz default-stack-vals)
          ;; Restore EDI, and reset the stack.
          (emit-label restore-edi)
          (loadw edi-tn ebx-tn (frame-word-offset (+ sp->fp-offset 1)))
-         (inst mov esp-tn ebx-tn)
-         (inst cld)))))
+         (inst mov esp-tn ebx-tn)))))
   (values))
 
 ;;;; unknown values receiving
@@ -502,7 +509,7 @@
 ;;; them.)
 (defun receive-unknown-values (args nargs start count node)
   (declare (type tn args nargs start count))
-  (let ((type (sb!c::basic-combination-derived-type node))
+  (let ((type (sb-c::basic-combination-derived-type node))
         (variable-values (gen-label))
         (stack-values (gen-label))
         (done (gen-label)))
@@ -555,13 +562,13 @@
 (defun check-ocfp-and-return-pc (old-fp return-pc)
   #+nil
   (format t "*known-return: old-fp ~S, tn-kind ~S; ~S ~S~%"
-          old-fp (tn-kind old-fp) (sb!c::tn-save-tn old-fp)
-          (tn-kind (sb!c::tn-save-tn old-fp)))
+          old-fp (tn-kind old-fp) (sb-c::tn-save-tn old-fp)
+          (tn-kind (sb-c::tn-save-tn old-fp)))
   #+nil
   (format t "*known-return: return-pc ~S, tn-kind ~S; ~S ~S~%"
           return-pc (tn-kind return-pc)
-          (sb!c::tn-save-tn return-pc)
-          (tn-kind (sb!c::tn-save-tn return-pc)))
+          (sb-c::tn-save-tn return-pc)
+          (tn-kind (sb-c::tn-save-tn return-pc)))
   (unless (and (sc-is old-fp control-stack)
                (= (tn-offset old-fp) ocfp-save-offset))
     (error "ocfp not on stack in standard save location?"))
@@ -903,10 +910,10 @@
                          '(make-ea :dword :disp
                            (+ nil-value (static-fun-offset fun))))
                         ((t)
-                         '(make-ea-for-object-slot eax fdefn-raw-addr-slot
+                         '(object-slot-ea eax fdefn-raw-addr-slot
                            other-pointer-lowtag))
                         ((nil)
-                         '(make-ea-for-object-slot eax closure-fun-slot
+                         '(object-slot-ea eax closure-fun-slot
                            fun-pointer-lowtag))))
                ,@(ecase return
                    (:fixed
@@ -1123,7 +1130,8 @@
   (:generator 20
     ;; Avoid the copy if there are no more args.
     (cond ((zerop fixed)
-           (inst jecxz JUST-ALLOC-FRAME))
+           (inst test ecx-tn ecx-tn)
+           (inst jmp :z JUST-ALLOC-FRAME))
           (t
            (inst cmp ecx-tn (fixnumize fixed))
            (inst jmp :be JUST-ALLOC-FRAME)))
@@ -1264,8 +1272,8 @@
 
     DONE))
 
-(define-vop (more-kw-arg)
-  (:translate sb!c::%more-kw-arg)
+(define-vop ()
+  (:translate sb-c::%more-kw-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1))
          (index :scs (any-reg immediate) :to (:result 1) :target keyword))
@@ -1285,7 +1293,7 @@
                                   :disp n-word-bytes))))))
 
 (define-vop (more-arg/c)
-  (:translate sb!c::%more-arg)
+  (:translate sb-c::%more-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1)))
   (:info index)
@@ -1297,7 +1305,7 @@
                                     :disp (- (* index n-word-bytes))))))
 
 (define-vop (more-arg)
-    (:translate sb!c::%more-arg)
+    (:translate sb-c::%more-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1))
          (index :scs (any-reg) :to (:result 1) :target value))
@@ -1309,8 +1317,24 @@
     (inst neg value)
     (inst mov value (make-ea :dword :base object :index value))))
 
+(define-vop (more-arg-or-nil)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1))
+         (count :scs (any-reg) :to (:result 1)))
+  (:arg-types * tagged-num)
+  (:info index)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst mov value nil-value)
+    (inst cmp count (fixnumize index))
+    (inst jmp :be done)
+    (inst mov value (make-ea :dword :base object
+                                    :disp (- (* index n-word-bytes))))
+    done))
+
 ;;; Turn more arg (context, count) into a list.
-(define-vop (listify-rest-args)
+(define-vop ()
   (:translate %listify-rest-args)
   (:policy :safe)
   (:args (context :scs (descriptor-reg) :target src)
@@ -1331,32 +1355,31 @@
       (move ecx count)
       ;; Check to see whether there are no args, and just return NIL if so.
       (inst mov result nil-value)
-      (inst jecxz done)
+      (inst test ecx ecx)
+      (inst jmp :z done)
       (inst lea dst (make-ea :dword :base ecx :index ecx))
-      (maybe-pseudo-atomic stack-allocate-p
-       (allocation dst dst node stack-allocate-p list-pointer-lowtag)
-       ;; Set decrement mode (successive args at lower addresses)
-       (inst std)
-       ;; Set up the result.
-       (move result dst)
-       ;; Jump into the middle of the loop, 'cause that's where we want
-       ;; to start.
-       (inst jmp enter)
-       (emit-label loop)
-       ;; Compute a pointer to the next cons.
-       (inst add dst (* cons-size n-word-bytes))
-       ;; Store a pointer to this cons in the CDR of the previous cons.
-       (storew dst dst -1 list-pointer-lowtag)
-       (emit-label enter)
-       ;; Grab one value and stash it in the car of this cons.
-       (inst lods eax)
-       (storew eax dst 0 list-pointer-lowtag)
-       ;; Go back for more.
-       (inst sub ecx n-word-bytes)
-       (inst jmp :nz loop)
-       ;; NIL out the last cons.
-       (storew nil-value dst 1 list-pointer-lowtag)
-       (inst cld))
+      (pseudo-atomic (:elide-if stack-allocate-p)
+        (allocation 'list dst list-pointer-lowtag node stack-allocate-p dst)
+        ;; Set up the result.
+        (move result dst)
+        ;; Jump into the middle of the loop, 'cause that's where we want
+        ;; to start.
+        (inst jmp enter)
+        (emit-label loop)
+        ;; Compute a pointer to the next cons.
+        (inst add dst (* cons-size n-word-bytes))
+        ;; Store a pointer to this cons in the CDR of the previous cons.
+        (storew dst dst -1 list-pointer-lowtag)
+        (emit-label enter)
+        ;; Grab one value and stash it in the car of this cons.
+        (inst mov eax (make-ea :dword :base src))
+        (inst sub src n-word-bytes)
+        (storew eax dst 0 list-pointer-lowtag)
+        ;; Go back for more.
+        (inst sub ecx n-word-bytes)
+        (inst jmp :nz loop)
+        ;; NIL out the last cons.
+        (storew nil-value dst 1 list-pointer-lowtag))
       (emit-label done))))
 
 ;;; Return the location and size of the &MORE arg glob created by
@@ -1369,9 +1392,9 @@
 ;;; preventing this info from being returned as values. What we do is
 ;;; compute supplied - fixed, and return a pointer that many words
 ;;; below the current stack top.
-(define-vop (more-arg-context)
+(define-vop ()
   (:policy :fast-safe)
-  (:translate sb!c::%more-arg-context)
+  (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg) :target count))
   (:arg-types positive-fixnum (:constant fixnum))
   (:info fixed)
@@ -1427,16 +1450,16 @@
   ;; slot from a symbol would require an extra register on +SB-THREAD,
   ;; and reading a slot from a thread structure would require an extra
   ;; register on -SB-THREAD.
-  #!+sb-thread
+  #+sb-thread
   (progn
-    #!+win32 (inst push eax-tn)
-    (with-tls-ea (EA :base #!+win32 eax-tn #!-win32 :unused
+    #+win32 (inst push eax-tn)
+    (with-tls-ea (EA :base #+win32 eax-tn #-win32 :unused
                      :disp-type :constant
                      :disp (* thread-stepping-slot n-word-bytes))
       (inst cmp EA 0 :maybe-fs))
-    #!+win32 (inst pop eax-tn))
-  #!-sb-thread
-  (inst cmp (make-ea-for-symbol-value sb!impl::*stepping*) 0))
+    #+win32 (inst pop eax-tn))
+  #-sb-thread
+  (inst cmp (make-ea-for-symbol-value sb-impl::*stepping*) 0))
 
 (define-vop (step-instrument-before-vop)
   (:policy :fast-safe)
@@ -1446,4 +1469,4 @@
      (inst jmp :eq DONE)
      (inst break single-step-before-trap)
      DONE
-     (note-this-location vop :step-before-vop)))
+     (note-this-location vop :internal-error)))
