@@ -1203,6 +1203,9 @@
                      cell)))))
       (unless (slot-definition-class eslotd)
         (setf (slot-definition-class eslotd) class))
+      (let ((info (slot-definition-info eslotd)))
+        (setf (slot-info-allocation info) (slot-definition-allocation eslotd)
+              (slot-info-location info) (slot-definition-location eslotd)))
       (initialize-internal-slot-functions eslotd))))
 
 (defmethod compute-slots :around ((class standard-class))
@@ -1344,16 +1347,21 @@
                              'source source-location)))
 
 (defmethod remove-reader-method ((class slot-class) generic-function)
-  (let ((method (get-method generic-function () (list class) nil)))
+  (let ((method
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 1)
+              (get-method generic-function () (list class) nil))))
     (when method (remove-method generic-function method))))
 
 (defmethod remove-writer-method ((class slot-class) generic-function)
   (let ((method
-          (get-method generic-function () (list *the-class-t* class) nil)))
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 2)
+              (get-method generic-function () (list *the-class-t* class) nil))))
     (when method (remove-method generic-function method))))
 
 (defmethod remove-boundp-method ((class slot-class) generic-function)
-  (let ((method (get-method generic-function () (list class) nil)))
+  (let ((method
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 1)
+              (get-method generic-function () (list class) nil))))
     (when method (remove-method generic-function method))))
 
 ;;; MAKE-READER-METHOD-FUNCTION and MAKE-WRITER-METHOD-FUNCTION
@@ -1517,13 +1525,13 @@
 
 (defun %set-slot-value-checking-type (context slots slot value
                                       safe old-class new-class)
-  (setf (clos-slots-ref slots (slot-definition-location slot))
-        (if (and safe (not (unbound-marker-p value)))
-            (let ((name (slot-definition-name slot))
-                  (type (slot-definition-type slot)))
-              (%ensure-slot-value-type context name type value
-                                       old-class new-class))
-            value)))
+  (let ((value (if (and safe (not (unbound-marker-p value)))
+                   (let ((name (slot-definition-name slot))
+                         (type (slot-definition-type slot)))
+                     (%ensure-slot-value-type context name type value
+                                              old-class new-class))
+                   value)))
+    (setf (clos-slots-ref slots (slot-definition-location slot)) value)))
 
 (defvar *in-obsolete-instance-trap* nil)
 
@@ -1578,7 +1586,16 @@
          ;; Go through all the old local slots.
          (dolist (old old-instance-slots)
            (let* ((name (slot-definition-name old))
-                  (value (clos-slots-ref oslots (slot-definition-location old))))
+                  (cell (find-slot-cell owrapper name))
+                  (location (car cell))
+                  (value (cond
+                           ((fixnump location)
+                            (clos-slots-ref oslots location))
+                           ((not location)
+                            (let ((location (slot-info-location (cdr cell))))
+                              (aver (integerp location))
+                              (clos-slots-ref oslots (slot-info-location (cdr cell)))))
+                           (t (bug "non-FIXNUM non-NULL location in cell: ~S" cell)))))
              (unless (unbound-marker-p value)
                (let ((new (assq name layout)))
                  (cond (new
@@ -1652,9 +1669,17 @@
           (binding* ((cell (find-slot-cell old-wrapper (slot-definition-name new))
                            :exit-if-null)
                      (location (car cell))
-                     (value (if (fixnump location)
-                                (clos-slots-ref old-slots location)
-                                (cdr location))))
+                     (value (cond
+                              ((fixnump location)
+                               (clos-slots-ref old-slots location))
+                              ((not location)
+                               (let ((info (cdr cell)))
+                                 (case (slot-info-allocation info)
+                                   (:instance
+                                    (clos-slots-ref old-slots (slot-info-location info)))
+                                   (:class (cdr (slot-info-location info))))))
+                              (t
+                               (cdr location)))))
             (set-value value new)))))
 
     ;; Make the copy point to the old instance's storage, and make the
