@@ -40,7 +40,7 @@
 #include "validate.h"
 #include "gc-internal.h"
 #include "gc-private.h"
-#include "pseudo-atomic.h"
+#include "getallocptr.h"
 #include "code.h"
 
 #include <errno.h>
@@ -361,18 +361,20 @@ adjust_code_refs(struct heap_adjust __attribute__((unused)) *adj,
             FIXUP32(UNALIGNED_STORE32(fixup_where, adjusted), fixup_where);
     }
     sword_t displacement = (lispobj)code - original_vaddr;
-    if (!displacement) // if this code didn't move, do nothing
-        return;
     prev_loc = 0;
     while (varint_unpack(&unpacker, &loc) && loc != 0) {
         loc += prev_loc;
         prev_loc = loc;
         void* fixup_where = instructions + loc;
         int32_t rel32operand = UNALIGNED_LOAD32(fixup_where);
-        sword_t adjusted = (sword_t)rel32operand - displacement;
-        if (!(adjusted >= INT32_MIN && adjusted <= INT32_MAX))
+        int32_t abs_operand = (sword_t)fixup_where + 4 + rel32operand - displacement;
+        int32_t new_abs_operand = abs_operand + calc_adjustment(adj, abs_operand);
+        sword_t new_rel32operand = new_abs_operand - ((sword_t)fixup_where + 4);
+        // check for overflow before checking whether to write the new value
+        if (!(new_rel32operand >= INT32_MIN && new_rel32operand <= INT32_MAX))
             lose("Relative fixup @ %p exceeds 32 bits", fixup_where);
-        FIXUP_rel(UNALIGNED_STORE32(fixup_where, adjusted), fixup_where);
+        if (new_rel32operand != rel32operand)
+            FIXUP_rel(UNALIGNED_STORE32(fixup_where, new_rel32operand), fixup_where);
     }
 #endif
 }
@@ -461,7 +463,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 #if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
             // Fixup absolute jump table
             lispobj* jump_table = code_jumptable_start(code);
-            int count = jump_table ? *jump_table : 0;
+            int count = jumptable_count(jump_table);
             int i;
             for (i = 1; i < count; ++i) adjust_word_at(jump_table+i, adj);
 #endif
@@ -606,8 +608,11 @@ static void relocate_heap(struct heap_adjust* adj)
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     // Pointers within varyobj space to varyobj space do not need adjustment
     // so remove any delta before performing the relocation pass on this space.
-    if (lisp_code_in_elf())
+    // FIXME: this was probably for PIE mode, which doesn't work.
+    if (lisp_code_in_elf() && adj->range[2].delta != 0) {
+        lose("code-in-elf + PIE not supported yet\n");
         adj->range[2].delta = 0; // FIXME: isn't this this already the case?
+    }
     relocate_space(VARYOBJ_SPACE_START, varyobj_free_pointer, adj);
 #endif
 }
@@ -700,15 +705,15 @@ process_directory(int count, struct ndir_entry *entry,
         // (Rarely would a jmp indirection be used; maybe for newly compiled code?)
         lispobj* ptr = &lisp_linkage_values;
         gc_assert(ptr);
-        char *link_target = (char*)(intptr_t)LINKAGE_TABLE_SPACE_START;
+        int entry_index = 0;
         int count;
         extern int lisp_linkage_table_n_prelinked;
         count = lisp_linkage_table_n_prelinked = *ptr++;
-        for ( ; count-- ; link_target += LINKAGE_TABLE_ENTRY_SIZE ) {
+        for ( ; count-- ; entry_index++ ) {
             boolean datap = *ptr == (lispobj)-1; // -1 can't be a function address
             if (datap)
                 ++ptr;
-            arch_write_linkage_table_entry(link_target, (void*)*ptr++, datap);
+            arch_write_linkage_table_entry(entry_index, (void*)*ptr++, datap);
         }
 
         // unprotect the pages
@@ -767,7 +772,6 @@ process_directory(int count, struct ndir_entry *entry,
         }
         if (len != 0) {
             spaces[id].len = len;
-            uword_t __attribute__((unused)) aligned_start;
             // Try to map at address requested by the core file.
             size_t request = spaces[id].desired_size;
             int sub_2gb_flag = (request & 1);
@@ -780,11 +784,7 @@ process_directory(int count, struct ndir_entry *entry,
             else
 #endif
             if (request) {
-                // Avoid "Relocation of fixedobj space not supported" error by
-                // not passing relocatable flag for immobile space if text in ELF.
-                addr = (uword_t)os_validate(sub_2gb_flag ?
-                                            (lisp_code_in_elf() ? NOT_MOVABLE : MOVABLE_LOW) :
-                                            MOVABLE,
+                addr = (uword_t)os_validate(sub_2gb_flag ? MOVABLE_LOW : MOVABLE,
                                             (os_vm_address_t)addr, request);
                 if (!addr) {
                     lose("Can't allocate %#"OBJ_FMTX" bytes for space %ld",
@@ -805,33 +805,37 @@ process_directory(int count, struct ndir_entry *entry,
 #endif
             case DYNAMIC_CORE_SPACE_ID:
 #ifdef LISP_FEATURE_CHENEYGC
-                aligned_start = ALIGN_UP(addr, BACKEND_PAGE_BYTES);
-                /* Misalignment can happen only if page size exceeds OS page.
-                 * Drop one page to avoid overrunning the allocated space */
-                if (aligned_start > addr) // not page-aligned
-                    dynamic_space_size -= BACKEND_PAGE_BYTES;
-                DYNAMIC_0_SPACE_START = addr = aligned_start;
-                current_dynamic_space = (lispobj*)addr;
-                { // Request that much again now
+                {
+                  uword_t semispace_0_start = ALIGN_UP(addr, BACKEND_PAGE_BYTES);
+                  uword_t semispace_0_end = ALIGN_DOWN(addr + request, BACKEND_PAGE_BYTES);
+                  // assign to 'addr' too, because that's where we load the core file
+                  DYNAMIC_0_SPACE_START = addr = semispace_0_start;
+                  current_dynamic_space = (lispobj*)addr;
+                  // Request that much again now
 #ifdef LISP_FEATURE_ALPHA /* meaning: is this a 32-on-64 SBCL implementation */
                   uword_t addr1 = (uword_t)os_validate(MOVABLE_LOW, 0, request);
                   extern os_set_cheneygc_spaces(uword_t,uword_t);
                   os_set_cheneygc_spaces(addr, addr1);
 #else
                   uword_t addr1 = (uword_t)os_allocate(request);
+                  uword_t semispace_1_start = ALIGN_UP(addr1, BACKEND_PAGE_BYTES);
+                  uword_t semispace_1_end = ALIGN_DOWN(addr1 + request, BACKEND_PAGE_BYTES);
 #endif
-                  aligned_start = ALIGN_UP(addr1, BACKEND_PAGE_BYTES);
-                  if (aligned_start > addr) // same test again
-                      dynamic_space_size -= BACKEND_PAGE_BYTES;
-                  DYNAMIC_1_SPACE_START = aligned_start;
+                  DYNAMIC_1_SPACE_START = semispace_1_start;
+                  uword_t semispace_0_size = semispace_0_end - semispace_0_start;
+                  uword_t semispace_1_size = semispace_1_end - semispace_1_start;
+                  dynamic_space_size =
+                      semispace_0_size < semispace_1_size ? semispace_0_size : semispace_1_size;
                 }
-#else
-                aligned_start = ALIGN_UP(addr, GENCGC_CARD_BYTES);
+#else /* gencgc */
+                {
+                uword_t aligned_start = ALIGN_UP(addr, GENCGC_CARD_BYTES);
                 /* Misalignment can happen only if card size exceeds OS page.
                  * Drop one card to avoid overrunning the allocated space */
                 if (aligned_start > addr) // not card-aligned
                     dynamic_space_size -= GENCGC_CARD_BYTES;
                 DYNAMIC_SPACE_START = addr = aligned_start;
+                }
 #endif
                 break;
             }
@@ -914,9 +918,6 @@ process_directory(int count, struct ndir_entry *entry,
                    spaces[DYNAMIC_CORE_SPACE_ID].len);
 #  endif // LISP_FEATURE_GENCGC
     if (adj->range[0].delta | adj->range[1].delta | adj->range[2].delta) {
-        if (adj->range[1].delta && lisp_code_in_elf())
-            lose("Relocation of fixedobj space not supported with ELF core.\n\
-Please report this as a bug");
         relocate_heap(adj);
     }
 
@@ -1138,14 +1139,14 @@ static void graph_visit(lispobj __attribute__((unused)) referer,
             layout = instance_layout(obj);
             graph_visit(ptr, layout, seen);
             nwords = instance_length(*obj);
-            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            bitmap = LAYOUT(layout)->bitmap;
             for(i=1; i<=nwords; ++i)
                 if (layout_bitmap_logbitp(i-1, bitmap)) RECURSE(obj[i]);
             break;
         case FUNCALLABLE_INSTANCE_WIDETAG:
             layout = funinstance_layout(obj);
             graph_visit(ptr, layout, seen);
-            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            bitmap = LAYOUT(layout)->bitmap;
             nwords = SHORT_BOXED_NWORDS(*obj);
             // We don't need to scan the word at index 1 (the trampoline pointer)
             // because it either points to the FIN itself or to readonly space.
