@@ -72,6 +72,8 @@ boolean os_preinit(char *argv[], char *envp[]);
 #else
 #define os_preinit(dummy1,dummy2) (0)
 #endif
+void os_link_runtime();
+void os_unlink_runtime();
 
 /* Do anything we need to do when starting up the runtime environment
  * in this OS. */
@@ -101,8 +103,10 @@ extern void os_zero(os_vm_address_t addr, os_vm_size_t length);
  * or (x == bit) depending on the use-case */
 #define NOT_MOVABLE      0
 #define MOVABLE          1
-#define MOVABLE_LOW      2
+#define ALLOCATE_LOW     2
 #define IS_THREAD_STRUCT 4
+#define MOVABLE_LOW      (MOVABLE|ALLOCATE_LOW)
+#define IS_GUARD_PAGE    8
 extern os_vm_address_t os_validate(int movable,
                                    os_vm_address_t addr,
                                    os_vm_size_t len);
@@ -116,10 +120,10 @@ extern void os_invalidate(os_vm_address_t addr, os_vm_size_t len);
 
 /* This maps a file into memory, or calls lose(..) for various
  * failures. */
-extern void os_map(int fd,
-                   int offset,
-                   os_vm_address_t addr,
-                   os_vm_size_t len);
+extern void* load_core_bytes(int fd,
+                             int offset,
+                             os_vm_address_t addr,
+                             os_vm_size_t len);
 
 /* This presumably flushes the instruction cache, if that can be done
  * explicitly. (It doesn't seem to be an issue for the i386 port,
@@ -164,6 +168,13 @@ os_context_register_t *os_context_lr_addr(os_context_t *context);
  * system stack pointer for that context. */
 #ifdef ARCH_HAS_STACK_POINTER
 os_context_register_t *os_context_sp_addr(os_context_t *context);
+#if (defined(LISP_FEATURE_X86)||defined(LISP_FEATURE_X86_64)) && \
+    (defined(LISP_FEATURE_DARWIN)||defined(LISP_FEATURE_LINUX)||defined(LISP_FEATURE_WIN32))
+os_context_register_t *os_context_fp_addr(os_context_t *context);
+#define os_context_frame_pointer(context) *os_context_fp_addr(context)
+#else
+#define os_context_frame_pointer(context) 0
+#endif
 #endif
 /* Given a signal context, return the address for storage of the
  * signal mask for that context. */
