@@ -29,6 +29,7 @@
 
 #if defined(LISP_FEATURE_SB_THREAD)
 #define thread_self() pthread_self()
+#define thread_equal(a,b) pthread_equal(a,b)
 #define thread_kill pthread_kill
 
 #ifdef LISP_FEATURE_WIN32
@@ -41,14 +42,13 @@
 #define thread_mutex_unlock(l) pthread_mutex_unlock(l)
 #else
 #define thread_self() 0
+#define thread_equal(a,b) ((a)==(b))
 #define thread_kill kill_safely
 #define thread_sigmask sigprocmask
 #define thread_mutex_lock(l) 0
 #define thread_mutex_unlock(l) 0
 #endif
 
-void os_link_runtime();
-
 #if defined(LISP_FEATURE_SB_SAFEPOINT)
 
 typedef enum {
@@ -261,7 +261,16 @@ lowtag_of(lispobj obj)
 }
 
 static inline int
-widetag_of(lispobj obj)
+widetag_of(lispobj* obj)
+{
+#ifdef LISP_FEATURE_BIG_ENDIAN
+    return ((unsigned char*)obj)[N_WORD_BYTES-1];
+#else
+    return *(unsigned char*)obj;
+#endif
+}
+static inline int
+header_widetag(lispobj obj)
 {
     return obj & WIDETAG_MASK;
 }
@@ -283,19 +292,14 @@ static inline int functionp(lispobj obj) {
 }
 static inline int simple_vector_p(lispobj obj) {
     return lowtag_of(obj) == OTHER_POINTER_LOWTAG &&
-           widetag_of(*(lispobj*)(obj-OTHER_POINTER_LOWTAG)) == SIMPLE_VECTOR_WIDETAG;
+           widetag_of((lispobj*)(obj-OTHER_POINTER_LOWTAG)) == SIMPLE_VECTOR_WIDETAG;
 }
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-#define HEADER_VALUE_MASKED(x) (HeaderValue(x) & SHORT_HEADER_MAX_WORDS)
-#else
-#define HEADER_VALUE_MASKED(x) HeaderValue(x)
-#endif
-static inline uword_t instance_length(lispobj header)
+static inline int instance_length(lispobj header)
 {
-  return HEADER_VALUE_MASKED(header);
+  return HeaderValue(header) & SHORT_HEADER_MAX_WORDS;
 }
-#undef HEADER_VALUE_MASKED
+
 /* Define an assignable instance_layout() macro taking a native pointer */
 #ifndef LISP_FEATURE_COMPACT_INSTANCE_HEADER
 # define instance_layout(instance_ptr) (instance_ptr)[1]
@@ -325,12 +329,13 @@ is_lisp_pointer(lispobj obj)
 static inline int
 is_lisp_immediate(lispobj obj)
 {
+    int widetag;
     return (fixnump(obj)
-            || (widetag_of(obj) == CHARACTER_WIDETAG)
+            || ((widetag = header_widetag(obj)) == CHARACTER_WIDETAG)
 #if N_WORD_BITS == 64
-            || (widetag_of(obj) == SINGLE_FLOAT_WIDETAG)
+            || (widetag == SINGLE_FLOAT_WIDETAG)
 #endif
-            || (widetag_of(obj) == UNBOUND_MARKER_WIDETAG));
+            || (widetag == UNBOUND_MARKER_WIDETAG));
 }
 
 /* Convert from a lispobj with type bits to a native (ordinary
@@ -362,28 +367,7 @@ fixnum_value(lispobj n)
     return (sword_t)n >> N_FIXNUM_TAG_BITS;
 }
 
-// Return signed int in case something tries to compute the number of boxed
-// words excluding the header word itself using "code_header_words(header) - 1",
-// which, for a filler needs to come out as negative, not a huge positive.
-static inline sword_t
-code_header_words(lispobj header) // given header = code->header
-{
-#ifdef LISP_FEATURE_64_BIT
-    return header >> 32;
-#else
-    /* Mask out bits reserved for GC */
-    return HeaderValue(header & ~((1 << 31) | (1 << 30)));
-#endif
-}
-
 #include "align.h"
-static inline unsigned int
-code_unboxed_nwords(lispobj n) // given n = code->code_size
-{
-    // Return ceiling |N / N_WORD_BYTES|
-    // Cast out high 32 bits of code_size if lispobj is 64 bits.
-    return (fixnum_value((uint32_t)n) + (N_WORD_BYTES-1)) >> WORD_SHIFT;
-}
 
 #if defined(LISP_FEATURE_WIN32)
 /* KLUDGE: Avoid double definition of boolean by rpcndr.h included via
@@ -415,12 +399,12 @@ is_cons_half(lispobj obj)
      * other_immediate and are Lisp immediates can be half of a cons */
     return !other_immediate_lowtag_p(obj)
 #if N_WORD_BITS == 64
-        || ((uword_t)IMMEDIATE_WIDETAGS_MASK >> (widetag_of(obj) >> 2)) & 1;
+        || ((uword_t)IMMEDIATE_WIDETAGS_MASK >> (header_widetag(obj) >> 2)) & 1;
 #else
       /* The above bit-shifting approach is not applicable
        * since we can't employ a 64-bit unsigned integer constant. */
-      || widetag_of(obj) == CHARACTER_WIDETAG
-      || widetag_of(obj) == UNBOUND_MARKER_WIDETAG;
+      || header_widetag(obj) == CHARACTER_WIDETAG
+      || header_widetag(obj) == UNBOUND_MARKER_WIDETAG;
 #endif
 }
 
@@ -495,4 +479,56 @@ extern struct lisp_startup_options lisp_startup_options;
 #define NO_SANITIZE_MEMORY
 #endif
 
+/// Object sizing macros. One of these days I'd like to rationalize
+/// all our headers making them more intuitive as to which need be
+/// included to get access to various things.
+/// For the time being, this file makes as much sense as anything else.
+
+/// These sizing macros return the number of *payload* words,
+/// exclusive of the object header word. Payload length is always
+/// an odd number so that total word count is an even number.
+
+/* Each size category is designed to allow 1 bit for a GC mark bit,
+ * possibly some flag bits, and the payload length in words.
+ * There are three size categories for most non-vector objects,
+ * differing in how many flag bits versus size bits there are.
+ * The GC mark bit is always in bit index 31 of the header regardless of
+ * machine word size.  Bit index 31 is chosen for consistency between 32-bit
+ * and 64-bit machines. It is a natural choice for 32-bit headers by avoiding
+ * intererence with other header fields. It is also chosen for 64-bit headers
+ * because the upper 32 bits of headers for some objects are already occupied
+ * by other data: symbol TLS index, instance layout, etc.
+ */
+
+/* The largest payload count is expressed in 23 bits. These objects
+ * can't reside in immobile space as there is no room for generation bits.
+ * All sorts of objects fall into this category, but mostly due to inertia.
+ * There are no non-vector boxed objects whose size should be so large.
+ * Header:   size |    tag
+ *          -----   ------
+ *        23 bits | 8 bits
+ */
+#define BOXED_NWORDS(obj) ((HeaderValue(obj) & 0x7FFFFF) | 1)
+
+/* Medium-sized payload count is expressed in 15 bits. Objects in this category
+ * may reside in immobile space: CLOSURE, INSTANCE, FUNCALLABLE-INSTANCE.
+ * The single data bit is used as a closure's NAMED flag,
+ * or an instance's "special GC strategy" flag.
+ *
+ * Header:  gen# |  data |     size |    tag
+ *         -----   -----    -------   ------
+ *        8 bits | 1 bit |  15 bits | 8 bits
+ */
+#define SHORT_BOXED_NWORDS(obj) ((HeaderValue(obj) & SHORT_HEADER_MAX_WORDS) | 1)
+
+/* Tiny payload count is expressed in 8 bits. Objects in this size category
+ * can reside in immobile space: SYMBOL, FDEFN.
+ * Header:  gen# | flags |   size |    tag
+ *         -----   ------  ------   ------
+ *        8 bits   8 bits  8 bits | 8 bits
+ * FDEFN  flag bits: 1 bit for statically-linked
+ * SYMBOL flag bits: 1 bit for present in initial core image
+ */
+#define TINY_BOXED_NWORDS(obj) ((HeaderValue(obj) & 0xFF) | 1)
+
 #endif /* _SBCL_RUNTIME_H_ */
